# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/12/23 17:27
# Author     ：Maxwell
# Description：
"""

from loguru import logger
from .core import setup_logger, get_logger_manager, shutdown_logger
from .context import (
    set_request_id,
    get_request_id,
    set_user_id,
    get_user_id,
    set_trace_id,
    get_trace_id,
    set_span_id,
    get_span_id,
    set_context,
    get_context,
    get_all_context,
    clear_context,
    with_request_id,
    with_user_context,
)
from .metrics import get_metrics

__all__ = [
    # 日志对象
    "logger",

    # 核心功能
    "setup_logger",
    "get_logger_manager",
    "shutdown_logger",

    # 上下文管理
    "set_request_id",
    "get_request_id",
    "set_user_id",
    "get_user_id",
    "set_trace_id",
    "get_trace_id",
    "set_span_id",
    "get_span_id",
    "set_context",
    "get_context",
    "get_all_context",
    "clear_context",
    "with_request_id",
    "with_user_context",

    # 指标统计
    "get_metrics",
]



import importlib
from pathlib import Path


def auto_import_modules_from_folder(package: str, folder: str):
    for file in Path(folder).glob("*.py"):
        if file.name.startswith("_"):
            continue
        module_name = f"{package}.{file.stem}"
        importlib.import_module(module_name)



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/12/22 21:38
# Author     ：Maxwell
# Description：
"""
import os
from dotenv import load_dotenv
from functools import lru_cache
from pathlib import Path
from .base import BaseConfig
from .db_relation import DatabaseConfig
from .db_cache import RedisConfig
from .db_vector import VectorDBConfig
from .mq import MessageQueueConfig
from .jwt import JWTConfig
from .llm import LLMConfig
from .log import LogConfig
from pydantic_settings import SettingsConfigDict


ENV_FILE_MAP = {
    'dev': 'config/.env.dev',
    'test': 'config/.env.test',
    'prod': 'config/.env.prod',
}


load_dotenv()
ENV = os.getenv('ENV', 'dev')

if ENV not in ENV_FILE_MAP:
    raise ValueError(f"无效的环境变量 ENV={ENV}，有效值: {list(ENV_FILE_MAP.keys())}")

ENV_FILE = ENV_FILE_MAP[ENV]
if not Path(ENV_FILE).exists():
    raise FileNotFoundError(f"配置文件不存在: {ENV_FILE}")


class Settings(
    BaseConfig,
    DatabaseConfig,
    RedisConfig,
    VectorDBConfig,
    MessageQueueConfig,
    JWTConfig,
    LLMConfig,
    LogConfig
):
    model_config = SettingsConfigDict(
        env_file=ENV_FILE,
        env_file_encoding="utf-8",
        case_sensitive=True,
        extra="ignore",
    )


@lru_cache(maxsize=1)
def get_settings() -> Settings:
    return Settings()


settings = get_settings()



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
Qdrant 向量数据库后端

功能：
- 异步连接管理
- 健康检查
- 集合管理
- 向量操作
"""

from typing import Optional, Dict, Any, List
from qdrant_client import AsyncQdrantClient
from qdrant_client.models import Distance, VectorParams, PointStruct
from loguru import logger

from infoman.config import VectorDBConfig
from infoman.service.infrastructure.base import (
    BaseInfrastructureComponent,
    ComponentType,
    ComponentStatus,
)


class QdrantBackend(BaseInfrastructureComponent):
    """Qdrant 向量数据库后端"""

    def __init__(self, config: VectorDBConfig):
        super().__init__(
            component_type=ComponentType.OTHER,  # 向量数据库可以归类为 OTHER
            name="qdrant",
            enabled=config.qdrant_configured,
        )
        self.config = config
        self._client: Optional[AsyncQdrantClient] = None

    async def startup(self) -> bool:
        """启动 Qdrant 连接"""
        if not self.enabled:
            logger.info("⏭️ Qdrant 未启用，跳过初始化")
            self._set_status(ComponentStatus.NOT_CONFIGURED)
            return False

        try:
            self._set_status(ComponentStatus.INITIALIZING)
            logger.info(f"🔌 正在连接 Qdrant [{self.config.QDRANT_HOST}]...")

            # 创建 Qdrant 客户端
            self._client = AsyncQdrantClient(
                host=self.config.QDRANT_HOST,
                port=self.config.QDRANT_HTTP_PORT,
                grpc_port=self.config.QDRANT_GRPC_PORT,
                api_key=self.config.QDRANT_API_KEY,
                timeout=self.config.QDRANT_TIMEOUT,
                prefer_grpc=True,  # 优先使用 gRPC（性能更好）
            )

            # 测试连接
            await self._test_connection()

            self._set_status(ComponentStatus.HEALTHY)
            self._log_startup(
                True,
                f"{self.config.QDRANT_HOST}:{self.config.QDRANT_HTTP_PORT}"
            )

            return True

        except Exception as e:
            self._set_status(ComponentStatus.UNHEALTHY)
            self._log_startup(False, str(e))
            return False

    async def shutdown(self):
        """关闭 Qdrant 连接"""
        if not self._client:
            return

        try:
            logger.info("⏹️ 正在关闭 Qdrant 连接...")

            # Qdrant 客户端会自动清理连接
            # 如果需要显式关闭，可以添加逻辑
            self._client = None

            self._set_status(ComponentStatus.STOPPED)
            self._log_shutdown(True)

        except Exception as e:
            self._log_shutdown(False, str(e))

    async def health_check(self) -> Dict[str, Any]:
        """健康检查"""
        if not self.enabled:
            return {
                "status": "not_configured",
                "component_type": self.component_type.value,
                "name": self.name,
                "details": {"enabled": False}
            }

        if not self._client:
            return {
                "status": "unhealthy",
                "component_type": self.component_type.value,
                "name": self.name,
                "details": {"error": "客户端未初始化"}
            }

        try:
            # 尝试获取集合列表
            collections = await self._client.get_collections()

            return {
                "status": "healthy",
                "component_type": self.component_type.value,
                "name": self.name,
                "details": {
                    "host": self.config.QDRANT_HOST,
                    "http_port": self.config.QDRANT_HTTP_PORT,
                    "grpc_port": self.config.QDRANT_GRPC_PORT,
                    "collections_count": len(collections.collections),
                }
            }

        except Exception as e:
            logger.error(f"Qdrant 健康检查失败: {e}")
            self._set_status(ComponentStatus.UNHEALTHY)

            return {
                "status": "unhealthy",
                "component_type": self.component_type.value,
                "name": self.name,
                "details": {"error": str(e)}
            }

    async def _test_connection(self):
        """测试连接"""
        try:
            # 尝试获取集合列表
            await self._client.get_collections()
            logger.success("✅ Qdrant 连接测试成功")

        except Exception as e:
            logger.error(f"❌ Qdrant 连接测试失败: {e}")
            raise

    # ========== 便捷方法 ==========

    async def create_collection(
        self,
        collection_name: str,
        vector_size: int,
        distance: Distance = Distance.COSINE,
        **kwargs
    ) -> bool:
        """
        创建集合

        Args:
            collection_name: 集合名称
            vector_size: 向量维度
            distance: 距离度量（COSINE/EUCLID/DOT）
            **kwargs: 其他参数

        Returns:
            是否创建成功
        """
        if not self._client:
            logger.error("Qdrant 客户端未初始化")
            return False

        try:
            # 检查集合是否存在
            collections = await self._client.get_collections()
            collection_names = [c.name for c in collections.collections]

            if collection_name in collection_names:
                logger.warning(f"集合 {collection_name} 已存在")
                return True

            # 创建集合
            await self._client.create_collection(
                collection_name=collection_name,
                vectors_config=VectorParams(
                    size=vector_size,
                    distance=distance,
                ),
                **kwargs
            )

            logger.success(f"✅ 创建集合成功: {collection_name}")
            return True

        except Exception as e:
            logger.error(f"创建集合失败: {e}")
            return False

    async def delete_collection(self, collection_name: str) -> bool:
        """删除集合"""
        if not self._client:
            return False

        try:
            await self._client.delete_collection(collection_name)
            logger.success(f"✅ 删除集合成功: {collection_name}")
            return True

        except Exception as e:
            logger.error(f"删除集合失败: {e}")
            return False

    async def upsert_points(
        self,
        collection_name: str,
        points: List[PointStruct],
    ) -> bool:
        """插入或更新向量点"""
        if not self._client:
            return False

        try:
            await self._client.upsert(
                collection_name=collection_name,
                points=points,
            )
            logger.debug(f"✅ 插入/更新 {len(points)} 个点到 {collection_name}")
            return True

        except Exception as e:
            logger.error(f"插入向量失败: {e}")
            return False

    async def search(
        self,
        collection_name: str,
        query_vector: List[float],
        limit: int = 10,
        score_threshold: Optional[float] = None,
        **kwargs
    ) -> List[Dict[str, Any]]:
        """
        向量搜索

        Args:
            collection_name: 集合名称
            query_vector: 查询向量
            limit: 返回结果数量
            score_threshold: 相似度阈值
            **kwargs: 其他搜索参数

        Returns:
            搜索结果列表
        """
        if not self._client:
            return []

        try:
            results = await self._client.search(
                collection_name=collection_name,
                query_vector=query_vector,
                limit=limit,
                score_threshold=score_threshold,
                **kwargs
            )

            return [
                {
                    "id": r.id,
                    "score": r.score,
                    "payload": r.payload,
                }
                for r in results
            ]

        except Exception as e:
            logger.error(f"向量搜索失败: {e}")
            return []

    async def get_collection_info(self, collection_name: str) -> Optional[Dict[str, Any]]:
        """获取集合信息"""
        if not self._client:
            return None

        try:
            info = await self._client.get_collection(collection_name)
            return {
                "name": collection_name,
                "vectors_count": info.vectors_count,
                "points_count": info.points_count,
                "status": info.status,
            }

        except Exception as e:
            logger.error(f"获取集合信息失败: {e}")
            return None

    async def get_stats(self) -> Dict[str, Any]:
        """获取统计信息"""
        if not self._client or not self.is_available:
            return {}

        try:
            collections = await self._client.get_collections()

            stats = {
                "collections_count": len(collections.collections),
                "collections": []
            }

            for collection in collections.collections:
                info = await self.get_collection_info(collection.name)
                if info:
                    stats["collections"].append(info)

            return stats

        except Exception as e:
            logger.error(f"获取统计信息失败: {e}")
            return {}



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/12/23 18:02
# Author     ：Maxwell
# Description：
"""
import orjson
import time
from pathlib import Path
from typing import Dict, List, Optional
from threading import Thread, Lock, Event
from queue import Queue, Empty

try:
    import httpx

    HTTPX_AVAILABLE = True
except ImportError:
    HTTPX_AVAILABLE = False


class LokiHandler:
    """
    Loki 日志处理器

    功能：
    1. 批量推送（减少网络请求）
    2. 异步推送（不阻塞主线程）
    3. 自动重试（网络异常时）
    4. 降级策略（Loki 不可用时写备份文件）
    """

    def __init__(
            self,
            url: str,
            labels: Dict[str, str],
            batch_size: int = 100,
            flush_interval: float = 5.0,
            timeout: float = 10.0,
            retry_times: int = 3,
            retry_backoff: float = 2.0,
            enable_fallback: bool = True,
            fallback_file: Optional[Path] = None,
    ):
        """
        初始化 Loki 处理器

        Args:
            url: Loki 服务地址
            labels: Loki Labels
            batch_size: 批量推送大小
            flush_interval: 刷新间隔（秒）
            timeout: 推送超时（秒）
            retry_times: 重试次数
            retry_backoff: 重试退避系数
            enable_fallback: 启用降级策略
            fallback_file: 备份文件路径
        """
        if not HTTPX_AVAILABLE:
            raise ImportError("LokiHandler 需要 httpx: pip install httpx")

        self.url = f"{url}/loki/api/v1/push"
        self.labels = labels
        self.batch_size = batch_size
        self.flush_interval = flush_interval
        self.timeout = timeout
        self.retry_times = retry_times
        self.retry_backoff = retry_backoff
        self.enable_fallback = enable_fallback
        self.fallback_file = fallback_file

        # 日志队列
        self.queue: Queue = Queue(maxsize=10000)
        self.batch: List[Dict] = []
        self.lock = Lock()

        # 后台线程
        self.stop_event = Event()
        self.worker_thread = Thread(target=self._worker, daemon=True)
        self.worker_thread.start()

        # HTTP 客户端
        self.client = httpx.Client(timeout=timeout)

        # 统计信息
        self.sent_count = 0
        self.failed_count = 0
        self.fallback_count = 0

    def write(self, message):
        """
        写入日志（Loguru 调用）

        Args:
            message: 日志消息对象
        """
        try:
            record = message.record

            # 构造日志条目
            entry = {
                "timestamp": int(record["time"].timestamp() * 1e9),  # 纳秒时间戳
                "line": self._format_line(record),
            }

            # 放入队列
            self.queue.put_nowait(entry)

        except Exception as e:
            # 避免日志处理器本身抛出异常
            print(f"LokiHandler 写入失败: {e}")

    def _format_line(self, record: Dict) -> str:
        """格式化日志行"""
        return orjson.dumps({
            "level": record["level"].name,
            "logger": record["name"],
            "function": record["function"],
            "line": record["line"],
            "message": record["message"],
            "extra": record.get("extra", {}),
        }).decode('utf-8')

    def _worker(self):
        """后台工作线程"""
        last_flush_time = time.time()

        while not self.stop_event.is_set():
            try:
                # 从队列获取日志
                try:
                    entry = self.queue.get(timeout=1.0)

                    with self.lock:
                        self.batch.append(entry)

                except Empty:
                    pass

                # 检查是否需要刷新
                current_time = time.time()
                should_flush = (
                        len(self.batch) >= self.batch_size or
                        (self.batch and current_time - last_flush_time >= self.flush_interval)
                )

                if should_flush:
                    self._flush()
                    last_flush_time = current_time

            except Exception as e:
                print(f"LokiHandler 工作线程异常: {e}")

        # 线程退出前，刷新剩余日志
        self._flush()

    def _flush(self):
        """刷新日志批次"""
        with self.lock:
            if not self.batch:
                return

            batch_to_send = self.batch.copy()
            self.batch.clear()

        # 推送到 Loki
        success = self._push_to_loki(batch_to_send)

        if success:
            self.sent_count += len(batch_to_send)
        else:
            self.failed_count += len(batch_to_send)

            # 降级策略：写备份文件
            if self.enable_fallback:
                self._write_fallback(batch_to_send)

    def _push_to_loki(self, batch: List[Dict]) -> bool:
        """
        推送日志到 Loki

        Args:
            batch: 日志批次

        Returns:
            True: 成功
            False: 失败
        """
        # 构造 Loki 请求体
        payload = {
            "streams": [
                {
                    "stream": self.labels,
                    "values": [
                        [str(entry["timestamp"]), entry["line"]]
                        for entry in batch
                    ]
                }
            ]
        }

        # 重试推送
        for attempt in range(self.retry_times + 1):
            try:
                response = self.client.post(
                    self.url,
                    json=payload,
                    headers={"Content-Type": "application/json"},
                )

                if response.status_code == 204:
                    return True

                print(
                    f"Loki 推送失败 [尝试 {attempt + 1}/{self.retry_times + 1}]: "
                    f"HTTP {response.status_code} - {response.text}"
                )

            except Exception as e:
                print(
                    f"Loki 推送异常 [尝试 {attempt + 1}/{self.retry_times + 1}]: {e}"
                )

            # 退避重试
            if attempt < self.retry_times:
                time.sleep(self.retry_backoff ** attempt)

        return False

    def _write_fallback(self, batch: List[Dict]):
        """
        写入备份文件

        Args:
            batch: 日志批次
        """
        if not self.fallback_file:
            return

        try:
            with open(self.fallback_file, "a", encoding="utf-8") as f:
                for entry in batch:
                    f.write(entry["line"] + "\n")

            self.fallback_count += len(batch)

        except Exception as e:
            print(f"写入备份文件失败: {e}")

    def flush(self):
        """手动刷新"""
        self._flush()

    def close(self):
        """关闭处理器"""
        # 停止工作线程
        self.stop_event.set()
        self.worker_thread.join(timeout=5.0)

        # 关闭 HTTP 客户端
        self.client.close()

        print(
            f"LokiHandler 已关闭 "
            f"[发送={self.sent_count}, 失败={self.failed_count}, "
            f"备份={self.fallback_count}]"
        )

    def __del__(self):
        """析构函数"""
        try:
            self.close()
        except:
            pass



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/12/23 17:58
# Author     ：Maxwell
# Description：
"""
import contextvars
from typing import Dict, Any, Optional
from uuid import uuid4

# =================================================================
# 上下文变量
# =================================================================

# 请求 ID
request_id_var: contextvars.ContextVar[Optional[str]] = contextvars.ContextVar(
    'request_id', default=None
)

# 用户 ID
user_id_var: contextvars.ContextVar[Optional[str]] = contextvars.ContextVar(
    'user_id', default=None
)

# 追踪 ID（分布式追踪）
trace_id_var: contextvars.ContextVar[Optional[str]] = contextvars.ContextVar(
    'trace_id', default=None
)

# Span ID（分布式追踪）
span_id_var: contextvars.ContextVar[Optional[str]] = contextvars.ContextVar(
    'span_id', default=None
)

# 自定义上下文
custom_context_var: contextvars.ContextVar[Dict[str, Any]] = contextvars.ContextVar(
    'custom_context', default={}
)


# =================================================================
# 上下文管理函数
# =================================================================

def set_request_id(request_id: Optional[str] = None) -> str:
    """
    设置请求 ID

    Args:
        request_id: 请求 ID（None 则自动生成）

    Returns:
        请求 ID
    """
    if request_id is None:
        request_id = str(uuid4())

    request_id_var.set(request_id)
    return request_id


def get_request_id() -> Optional[str]:
    """获取请求 ID"""
    return request_id_var.get()


def set_user_id(user_id: str):
    """设置用户 ID"""
    user_id_var.set(user_id)


def get_user_id() -> Optional[str]:
    """获取用户 ID"""
    return user_id_var.get()


def set_trace_id(trace_id: str):
    """设置追踪 ID"""
    trace_id_var.set(trace_id)


def get_trace_id() -> Optional[str]:
    """获取追踪 ID"""
    return trace_id_var.get()


def set_span_id(span_id: str):
    """设置 Span ID"""
    span_id_var.set(span_id)


def get_span_id() -> Optional[str]:
    """获取 Span ID"""
    return span_id_var.get()


def set_context(key: str, value: Any):
    """
    设置自定义上下文

    Args:
        key: 键
        value: 值
    """
    context = custom_context_var.get().copy()
    context[key] = value
    custom_context_var.set(context)


def get_context(key: str) -> Optional[Any]:
    """获取自定义上下文"""
    return custom_context_var.get().get(key)


def get_all_context() -> Dict[str, Any]:
    """获取所有上下文"""
    context = {
        "request_id": get_request_id(),
        "user_id": get_user_id(),
        "trace_id": get_trace_id(),
        "span_id": get_span_id(),
    }

    # 合并自定义上下文
    context.update(custom_context_var.get())

    # 移除 None 值
    return {k: v for k, v in context.items() if v is not None}


def clear_context():
    """清除所有上下文"""
    request_id_var.set(None)
    user_id_var.set(None)
    trace_id_var.set(None)
    span_id_var.set(None)
    custom_context_var.set({})


# =================================================================
# 上下文装饰器
# =================================================================

from functools import wraps


def with_request_id(func):
    """
    自动设置请求 ID 的装饰器

    用法:
        @with_request_id
        def my_function():
            logger.info("处理请求")  # 自动包含 request_id
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        set_request_id()
        try:
            return func(*args, **kwargs)
        finally:
            clear_context()

    return wrapper


def with_user_context(user_id: str):
    """
    自动设置用户上下文的装饰器

    用法:
        @with_user_context("user_123")
        def my_function():
            logger.info("用户操作")  # 自动包含 user_id
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            set_user_id(user_id)
            try:
                return func(*args, **kwargs)
            finally:
                clear_context()

        return wrapper

    return decorator



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2023/12/9 13:39
# Author     ：Maxwell
# Description：
"""

from fastapi import Request, APIRouter

from infoman.service.core.response import success

health_router = APIRouter(prefix="")


@health_router.get(summary="", path="")
async def api_health(req: Request):
    return success()



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/12/22 21:11
# Author     ：Maxwell
# Description：
"""
from typing import Dict, Any
from infoman.config import DatabaseConfig


class SQLiteBackend:
    """SQLite 后端"""

    @staticmethod
    def get_engine() -> str:
        return "tortoise.backends.sqlite"

    @staticmethod
    def get_credentials(config: DatabaseConfig) -> Dict[str, Any]:
        """生成 SQLite 连接凭证"""
        return {
            "file_path": f"{config.database}.db",
        }



"""
混合数据库管理器 - 同时支持 Tortoise 和 SQLAlchemy

Version: 0.3.0
Author: Maxwell
"""

from typing import Dict, Optional, List, Any, Literal
from fastapi import FastAPI
from loguru import logger

from infoman.config import settings, DatabaseConfig

# ==================== 类型定义 ====================

ORMType = Literal["tortoise", "sqlalchemy"]


# ==================== 后端实现导入 ====================

# Tortoise 后端
try:
    from tortoise import Tortoise
    from infoman.service.infrastructure.db_relation.mysql import MySQLBackend
    from infoman.service.infrastructure.db_relation.sqllite import SQLiteBackend
    from infoman.service.infrastructure.db_relation.pgsql import PostgreSQLBackend

    TORTOISE_BACKENDS = {
        "mysql": MySQLBackend,
        "postgresql": PostgreSQLBackend,
        "sqlite": SQLiteBackend,
    }
    _TORTOISE_AVAILABLE = True
except ImportError:
    _TORTOISE_AVAILABLE = False
    TORTOISE_BACKENDS = {}

# SQLAlchemy 后端
try:
    from sqlalchemy.ext.asyncio import create_async_engine, async_sessionmaker, AsyncSession
    from sqlalchemy import text

    _SQLALCHEMY_AVAILABLE = True
except ImportError:
    _SQLALCHEMY_AVAILABLE = False


# ==================== SQLAlchemy 后端实现 ====================

class SQLAlchemyMySQLBackend:
    """SQLAlchemy MySQL 后端"""

    @staticmethod
    def get_url(config: DatabaseConfig) -> str:
        """生成 MySQL 连接 URL"""
        return (
            f"mysql+asyncmy://{config.user}:{config.password}@"
            f"{config.host}:{config.port}/{config.database}"
            f"?charset={config.charset}"
        )


class SQLAlchemyPostgreSQLBackend:
    """SQLAlchemy PostgreSQL 后端"""

    @staticmethod
    def get_url(config: DatabaseConfig) -> str:
        """生成 PostgreSQL 连接 URL"""
        return (
            f"postgresql+asyncpg://{config.user}:{config.password}@"
            f"{config.host}:{config.port}/{config.database}"
        )


class SQLAlchemySQLiteBackend:
    """SQLAlchemy SQLite 后端"""

    @staticmethod
    def get_url(config: DatabaseConfig) -> str:
        """生成 SQLite 连接 URL"""
        return f"sqlite+aiosqlite:///{config.database}"


SQLALCHEMY_BACKENDS = {
    "mysql": SQLAlchemyMySQLBackend,
    "postgresql": SQLAlchemyPostgreSQLBackend,
    "sqlite": SQLAlchemySQLiteBackend,
}


# ==================== 混合数据库管理器 ====================

class HybridDatabaseManager:
    """
    混合数据库管理器 - 同时支持 Tortoise 和 SQLAlchemy

    功能：
    - 支持多个数据库同时连接
    - 每个连接可独立选择 ORM（Tortoise 或 SQLAlchemy）
    - 统一的健康检查
    - 优雅关闭

    使用示例：
        >>> manager = HybridDatabaseManager()
        >>> await manager.startup(app)
        >>>
        >>> # 获取 Tortoise 连接
        >>> tortoise_conn = manager.get_tortoise_connection("default")
        >>>
        >>> # 获取 SQLAlchemy session maker
        >>> session_maker = manager.get_sqlalchemy_session_maker("analytics")
    """

    def __init__(self):
        # Tortoise 连接
        self.tortoise_connections: Dict[str, DatabaseConfig] = {}
        self.tortoise_initialized = False

        # SQLAlchemy 引擎
        self.sqlalchemy_engines: Dict[str, Any] = {}
        self.sqlalchemy_sessions: Dict[str, Any] = {}
        self.sqlalchemy_initialized = False

    @property
    def is_available(self) -> bool:
        """是否有可用的数据库连接"""
        return (
            (self.tortoise_initialized and len(self.tortoise_connections) > 0) or
            (self.sqlalchemy_initialized and len(self.sqlalchemy_engines) > 0)
        )

    @property
    def tortoise_client(self):
        """获取 Tortoise connections 对象"""
        if not self.tortoise_initialized:
            return None
        if not _TORTOISE_AVAILABLE:
            return None
        return Tortoise.connections

    def get_tortoise_connection(self, name: str = "default"):
        """
        获取 Tortoise 连接

        Args:
            name: 连接名称

        Returns:
            Tortoise connection 对象

        Raises:
            RuntimeError: Tortoise 未初始化
        """
        if not self.tortoise_initialized:
            raise RuntimeError("Tortoise ORM 未初始化")
        if not _TORTOISE_AVAILABLE:
            raise RuntimeError("Tortoise ORM 未安装")
        return Tortoise.connections.get(name)

    def get_sqlalchemy_session_maker(self, name: str = "default"):
        """
        获取 SQLAlchemy Session Maker

        Args:
            name: 连接名称

        Returns:
            async_sessionmaker 实例

        Raises:
            RuntimeError: SQLAlchemy 未初始化
        """
        if not self.sqlalchemy_initialized:
            raise RuntimeError("SQLAlchemy 未初始化")
        return self.sqlalchemy_sessions.get(name)

    def get_sqlalchemy_engine(self, name: str = "default"):
        """获取 SQLAlchemy Engine"""
        if not self.sqlalchemy_initialized:
            raise RuntimeError("SQLAlchemy 未初始化")
        return self.sqlalchemy_engines.get(name)

    def _classify_connections(self) -> tuple[Dict, Dict]:
        """
        分类数据库连接配置

        Returns:
            (tortoise_configs, sqlalchemy_configs)
        """
        enabled_dbs = settings.enabled_databases

        tortoise_configs = {}
        sqlalchemy_configs = {}

        for conn_name, db_config in enabled_dbs.items():
            # 根据配置或命名规则判断 ORM 类型
            orm_type = getattr(db_config, 'orm', None)

            # 如果未指定，使用命名规则判断
            if orm_type is None:
                if conn_name.endswith('_alchemy') or conn_name.endswith('_sqlalchemy'):
                    orm_type = "sqlalchemy"
                else:
                    orm_type = "tortoise"  # 默认使用 Tortoise（向前兼容）

            if orm_type == "tortoise":
                tortoise_configs[conn_name] = db_config
            elif orm_type == "sqlalchemy":
                sqlalchemy_configs[conn_name] = db_config
            else:
                logger.warning(f"⚠️  未知的 ORM 类型: {orm_type}，跳过连接 {conn_name}")

        return tortoise_configs, sqlalchemy_configs

    def _gen_tortoise_config(self, configs: Dict[str, DatabaseConfig]) -> Optional[dict]:
        """生成 Tortoise 配置"""
        if not configs:
            return None

        if not _TORTOISE_AVAILABLE:
            logger.warning("⚠️  Tortoise ORM 未安装，跳过 Tortoise 连接")
            return None

        connections = {}

        for conn_name, db_config in configs.items():
            backend_class = TORTOISE_BACKENDS.get(db_config.type)

            if not backend_class:
                logger.error(f"❌ 不支持的数据库类型: {db_config.type}")
                continue

            connections[conn_name] = {
                "engine": backend_class.get_engine(),
                "credentials": backend_class.get_credentials(db_config),
            }

            self.tortoise_connections[conn_name] = db_config

        if not connections:
            return None

        # 应用配置（模型分组）
        apps = {}
        for conn_name, db_config in configs.items():
            if db_config.models:
                app_name = f"{conn_name}_models"
                apps[app_name] = {
                    "models": db_config.models,
                    "default_connection": conn_name,
                }

        if not apps:
            return None

        config = {
            "connections": connections,
            "apps": apps,
            "use_tz": settings.DB_USE_TZ,
            "timezone": settings.DB_TIMEZONE,
        }

        return config

    async def _init_tortoise(self, app: Optional[FastAPI], configs: Dict[str, DatabaseConfig]):
        """初始化 Tortoise ORM"""
        if not configs:
            return

        config = self._gen_tortoise_config(configs)
        if not config:
            return

        try:
            logger.info("🚀 初始化 Tortoise ORM...")
            for conn_name, db_config in configs.items():
                logger.info(
                    f"   - [{conn_name}] {db_config.type.upper()}: "
                    f"{db_config.user}@{db_config.host}:{db_config.port}/{db_config.database}"
                )

            await Tortoise.init(config=config)

            if app:
                app.state.db_tortoise = self.tortoise_client
                logger.debug("✅ Tortoise 客户端已挂载到 app.state")

            self.tortoise_initialized = True
            logger.success(
                f"✅ Tortoise ORM 连接成功（{len(configs)} 个）\n"
                f"   连接名: {list(configs.keys())}"
            )

        except Exception as e:
            logger.error(f"❌ Tortoise ORM 连接失败: {e}")
            raise

    async def _init_sqlalchemy(self, app: Optional[FastAPI], configs: Dict[str, DatabaseConfig]):
        """初始化 SQLAlchemy"""
        if not configs:
            return

        if not _SQLALCHEMY_AVAILABLE:
            logger.warning("⚠️  SQLAlchemy 未安装，跳过 SQLAlchemy 连接")
            return

        try:
            logger.info("🚀 初始化 SQLAlchemy...")

            for conn_name, db_config in configs.items():
                backend_class = SQLALCHEMY_BACKENDS.get(db_config.type)

                if not backend_class:
                    logger.error(f"❌ 不支持的数据库类型: {db_config.type}")
                    continue

                url = backend_class.get_url(db_config)

                logger.info(
                    f"   - [{conn_name}] {db_config.type.upper()}: "
                    f"{db_config.user}@{db_config.host}:{db_config.port}/{db_config.database}"
                )

                engine = create_async_engine(
                    url,
                    pool_size=db_config.pool_max_size,
                    max_overflow=db_config.pool_max_size,
                    pool_recycle=db_config.pool_recycle,
                    echo=db_config.echo,
                    pool_pre_ping=True,  # 健康检查
                )

                session_maker = async_sessionmaker(
                    engine,
                    class_=AsyncSession,
                    expire_on_commit=False,
                )

                self.sqlalchemy_engines[conn_name] = engine
                self.sqlalchemy_sessions[conn_name] = session_maker

            if self.sqlalchemy_engines:
                if app:
                    app.state.db_sqlalchemy_engines = self.sqlalchemy_engines
                    app.state.db_sqlalchemy_sessions = self.sqlalchemy_sessions
                    logger.debug("✅ SQLAlchemy 引擎已挂载到 app.state")

                self.sqlalchemy_initialized = True
                logger.success(
                    f"✅ SQLAlchemy 连接成功（{len(self.sqlalchemy_engines)} 个）\n"
                    f"   连接名: {list(self.sqlalchemy_engines.keys())}"
                )

        except Exception as e:
            logger.error(f"❌ SQLAlchemy 连接失败: {e}")
            raise

    async def startup(self, app: Optional[FastAPI] = None) -> bool:
        """
        启动数据库管理器

        Args:
            app: FastAPI 应用实例（可选）

        Returns:
            是否成功初始化
        """
        if self.tortoise_initialized or self.sqlalchemy_initialized:
            logger.warning("⚠️  DatabaseManager 已初始化，跳过重复初始化")
            return True

        # 分类连接配置
        tortoise_configs, sqlalchemy_configs = self._classify_connections()

        if not tortoise_configs and not sqlalchemy_configs:
            logger.info("⏭️  数据库未配置，跳过初始化")
            return False

        # 初始化 Tortoise
        if tortoise_configs:
            await self._init_tortoise(app, tortoise_configs)

        # 初始化 SQLAlchemy
        if sqlalchemy_configs:
            await self._init_sqlalchemy(app, sqlalchemy_configs)

        return True

    async def register(self, app: FastAPI) -> bool:
        """注册到 FastAPI 应用"""
        return await self.startup(app)

    async def health_check(self, conn_name: Optional[str] = None) -> Dict:
        """
        健康检查

        Args:
            conn_name: 连接名称（可选，默认检查所有）

        Returns:
            健康检查结果
        """
        if not self.is_available:
            return {
                "status": "not_configured",
                "name": "database",
                "details": {"enabled": False}
            }

        results = {}

        # 检查 Tortoise 连接
        if self.tortoise_initialized:
            for name in self.tortoise_connections.keys():
                if conn_name is None or conn_name == name:
                    results[f"tortoise_{name}"] = await self._check_tortoise_connection(name)

        # 检查 SQLAlchemy 连接
        if self.sqlalchemy_initialized:
            for name in self.sqlalchemy_engines.keys():
                if conn_name is None or conn_name == name:
                    results[f"sqlalchemy_{name}"] = await self._check_sqlalchemy_connection(name)

        # 汇总状态
        all_healthy = all(r.get("status") == "healthy" for r in results.values())

        return {
            "status": "healthy" if all_healthy else "unhealthy",
            "name": "database",
            "details": {
                "connections": results,
                "count": len(results)
            }
        }

    async def _check_tortoise_connection(self, conn_name: str) -> Dict:
        """检查 Tortoise 连接"""
        try:
            conn = Tortoise.connections.get(conn_name)
            db_config = self.tortoise_connections.get(conn_name)

            if not conn or not db_config:
                return {
                    "status": "not_found",
                    "error": f"连接 '{conn_name}' 不存在",
                }

            # 执行健康检查查询
            await conn.execute_query("SELECT 1")

            # 获取连接池状态
            pool_status = {
                "size": conn._pool.size() if hasattr(conn, "_pool") else "N/A",
                "free": conn._pool.freesize() if hasattr(conn, "_pool") else "N/A",
            }

            return {
                "status": "healthy",
                "orm": "tortoise",
                "type": db_config.type,
                "database": db_config.database,
                "pool": pool_status,
            }

        except Exception as e:
            logger.error(f"❌ Tortoise 连接 '{conn_name}' 健康检查失败: {e}")
            return {
                "status": "unhealthy",
                "orm": "tortoise",
                "error": str(e),
            }

    async def _check_sqlalchemy_connection(self, conn_name: str) -> Dict:
        """检查 SQLAlchemy 连接"""
        try:
            engine = self.sqlalchemy_engines.get(conn_name)
            session_maker = self.sqlalchemy_sessions.get(conn_name)

            if not engine or not session_maker:
                return {
                    "status": "not_found",
                    "error": f"连接 '{conn_name}' 不存在",
                }

            # 执行健康检查查询
            async with session_maker() as session:
                result = await session.execute(text("SELECT 1"))
                result.scalar()

            # 获取连接池状态
            pool = engine.pool
            pool_status = {
                "size": pool.size(),
                "checked_in": pool.checkedin(),
                "checked_out": pool.checkedout(),
                "overflow": pool.overflow(),
            }

            return {
                "status": "healthy",
                "orm": "sqlalchemy",
                "type": str(engine.url.drivername),
                "database": engine.url.database,
                "pool": pool_status,
            }

        except Exception as e:
            logger.error(f"❌ SQLAlchemy 连接 '{conn_name}' 健康检查失败: {e}")
            return {
                "status": "unhealthy",
                "orm": "sqlalchemy",
                "error": str(e),
            }

    async def shutdown(self):
        """关闭所有数据库连接"""
        # 关闭 Tortoise
        if self.tortoise_initialized:
            try:
                logger.info("⏹️  关闭 Tortoise ORM 连接...")
                for conn_name in self.tortoise_connections.keys():
                    logger.info(f"   - 关闭连接: {conn_name}")

                await Tortoise.close_connections()
                self.tortoise_initialized = False
                logger.success("✅ Tortoise ORM 连接已关闭")

            except Exception as e:
                logger.error(f"❌ 关闭 Tortoise ORM 连接失败: {e}")

        # 关闭 SQLAlchemy
        if self.sqlalchemy_initialized:
            try:
                logger.info("⏹️  关闭 SQLAlchemy 连接...")
                for conn_name, engine in self.sqlalchemy_engines.items():
                    logger.info(f"   - 关闭引擎: {conn_name}")
                    await engine.dispose()

                self.sqlalchemy_engines.clear()
                self.sqlalchemy_sessions.clear()
                self.sqlalchemy_initialized = False
                logger.success("✅ SQLAlchemy 连接已关闭")

            except Exception as e:
                logger.error(f"❌ 关闭 SQLAlchemy 连接失败: {e}")

    async def close(self):
        """关闭所有数据库连接（兼容旧接口）"""
        await self.shutdown()


# ==================== 全局单例 ====================

hybrid_db_manager = HybridDatabaseManager()


# ==================== 便捷函数 ====================

async def register_databases(app: FastAPI) -> bool:
    """注册数据库到 FastAPI 应用"""
    return await hybrid_db_manager.register(app)


async def check_databases_health(conn_name: Optional[str] = None) -> Dict:
    """检查数据库健康状态"""
    return await hybrid_db_manager.health_check(conn_name)


async def close_databases():
    """关闭所有数据库连接"""
    await hybrid_db_manager.close()


def get_connection_names() -> Dict[str, List[str]]:
    """获取所有连接名称（按 ORM 分类）"""
    return {
        "tortoise": list(hybrid_db_manager.tortoise_connections.keys()),
        "sqlalchemy": list(hybrid_db_manager.sqlalchemy_engines.keys()),
    }



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/12/22 21:38
# Author     ：Maxwell
# Description：
"""
from typing import Optional
from pydantic import Field
from pydantic_settings import BaseSettings


class VectorDBConfig(BaseSettings):
    # ========== Qdrant ==========
    QDRANT_ENABLED: bool = Field(default=False, description="是否启用 Qdrant")
    QDRANT_HOST: Optional[str] = Field(default=None, description="Qdrant 主机")
    QDRANT_HTTP_PORT: int = Field(default=6333, description="HTTP 端口")
    QDRANT_GRPC_PORT: int = Field(default=6334, description="gRPC 端口")
    QDRANT_API_KEY: Optional[str] = Field(default=None, description="API Key")
    QDRANT_TIMEOUT: int = Field(default=10, description="超时时间（秒）")

    # ========== Milvus ==========
    MILVUS_ENABLED: bool = Field(default=False, description="是否启用 Milvus")
    MILVUS_HOST: Optional[str] = Field(default=None, description="Milvus 主机")
    MILVUS_PORT: int = Field(default=19530, description="Milvus 端口")
    MILVUS_USER: Optional[str] = Field(default=None, description="用户名")
    MILVUS_PASSWORD: Optional[str] = Field(default=None, description="密码")

    @property
    def qdrant_configured(self) -> bool:
        """Qdrant 是否已配置"""
        return self.QDRANT_ENABLED and bool(self.QDRANT_HOST)

    @property
    def milvus_configured(self) -> bool:
        """Milvus 是否已配置"""
        return self.MILVUS_ENABLED and bool(self.MILVUS_HOST)




# database/backends.py
"""
数据库后端实现
"""

from typing import Dict, Any
from infoman.config import DatabaseInstanceConfig


class MySQLBackend:
    """MySQL 后端"""

    @staticmethod
    def get_engine() -> str:
        return "tortoise.backends.mysql"

    @staticmethod
    def get_credentials(config: DatabaseInstanceConfig) -> Dict[str, Any]:
        """生成 MySQL 连接凭证"""
        credentials = {
            "host": config.host,
            "port": config.port,
            "user": config.user,
            "password": config.password,
            "database": config.database,
            "charset": config.charset,
            # 连接池
            "minsize": config.pool_min_size,
            "maxsize": config.pool_max_size,
            "pool_recycle": config.pool_recycle,
            "echo": config.echo,
            # 超时
            "connect_timeout": config.connect_timeout,
            # 其他
            "autocommit": True,
            "init_command": "SET sql_mode='STRICT_TRANS_TABLES'",
        }

        # SSL
        if config.ssl_enabled:
            ssl_config = {}
            if config.ssl_ca:
                ssl_config["ca"] = config.ssl_ca
            if config.ssl_cert:
                ssl_config["cert"] = config.ssl_cert
            if config.ssl_key:
                ssl_config["key"] = config.ssl_key

            if ssl_config:
                credentials["ssl"] = ssl_config

        return credentials



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/12/22 21:37
# Author     ：Maxwell
# Description：
"""
from typing import Optional, Literal
from pydantic import Field, field_validator
from pydantic_settings import BaseSettings


class RedisConfig(BaseSettings):
    """Redis/DragonflyDB 缓存配置"""

    # ========== 基础配置 ==========
    REDIS_ENABLED: bool = Field(
        default=False,
        description="是否启用缓存服务"
    )

    REDIS_BACKEND: Literal["redis", "dragonfly", "auto"] = Field(
        default="auto",
        description="缓存后端类型：redis/dragonfly/auto（自动检测）"
    )

    REDIS_HOST: Optional[str] = Field(
        default=None,
        description="缓存服务器主机地址"
    )

    REDIS_PORT: int = Field(
        default=6379,
        description="缓存服务器端口"
    )

    REDIS_DB: int = Field(
        default=0,
        ge=0,
        le=15,
        description="Redis 数据库编号（0-15）"
    )

    REDIS_PASSWORD: Optional[str] = Field(
        default=None,
        description="缓存服务器密码"
    )

    # ========== 连接池配置 ==========
    REDIS_MAX_CONNECTIONS: int = Field(
        default=30,
        ge=1,
        le=1000,
        description="连接池最大连接数"
    )

    REDIS_SOCKET_TIMEOUT: int = Field(
        default=4,
        ge=1,
        description="Socket 超时时间（秒）"
    )

    REDIS_SOCKET_CONNECT_TIMEOUT: int = Field(
        default=2,
        ge=1,
        description="连接超时时间（秒）"
    )

    REDIS_HEALTH_CHECK_INTERVAL: int = Field(
        default=30,
        ge=0,
        description="健康检查间隔（秒），0 表示禁用"
    )

    REDIS_RETRY_ON_TIMEOUT: bool = Field(
        default=True,
        description="超时时是否自动重试"
    )

    # ========== 缓存配置 ==========
    REDIS_CACHE_PREFIX: str = Field(
        default="infoman",
        description="缓存 key 前缀"
    )

    REDIS_CACHE_EXPIRE: int = Field(
        default=3600,
        ge=0,
        description="默认缓存过期时间（秒），0 表示永不过期"
    )

    REDIS_CACHE_VERSION: Optional[str] = Field(
        default=None,
        description="缓存版本号（用于缓存失效），留空则使用应用版本"
    )

    # ========== 编码配置 ==========
    REDIS_ENCODING: str = Field(
        default="utf-8",
        description="编码格式"
    )

    REDIS_DECODE_RESPONSES: bool = Field(
        default=False,
        description="是否自动解码响应（True: 返回 str，False: 返回 bytes）"
    )

    # ========== 验证器 ==========

    @field_validator("REDIS_BACKEND")
    @classmethod
    def validate_backend(cls, v: str) -> str:
        """验证后端类型"""
        allowed = ["redis", "dragonfly", "auto"]
        if v not in allowed:
            raise ValueError(f"REDIS_BACKEND 必须是 {allowed} 之一")
        return v

    @field_validator("REDIS_CACHE_PREFIX")
    @classmethod
    def validate_prefix(cls, v: str) -> str:
        if not v or not v.strip():
            raise ValueError("REDIS_CACHE_PREFIX 不能为空")
        return v.strip()

    @field_validator("REDIS_HOST")
    @classmethod
    def validate_host(cls, v: Optional[str]) -> Optional[str]:
        """验证主机地址"""
        if v:
            v = v.strip()
            if not v:
                return None
        return v

    # ========== 计算属性 ==========

    @property
    def redis_configured(self) -> bool:
        return self.REDIS_ENABLED and bool(self.REDIS_HOST)

    @property
    def is_redis(self) -> bool:
        """是否使用 Redis"""
        return self.REDIS_BACKEND == "redis"

    @property
    def is_dragonfly(self) -> bool:
        """是否使用 DragonflyDB"""
        return self.REDIS_BACKEND == "dragonfly"

    @property
    def is_auto(self) -> bool:
        """是否自动检测"""
        return self.REDIS_BACKEND == "auto"

    @property
    def connection_url(self) -> Optional[str]:
        """
        获取连接 URL

        Returns:
            Redis 连接 URL，例如：redis://:password@localhost:6379/0
        """
        if not self.redis_configured:
            return None

        auth = f":{self.REDIS_PASSWORD}@" if self.REDIS_PASSWORD else ""
        return f"redis://{auth}{self.REDIS_HOST}:{self.REDIS_PORT}/{self.REDIS_DB}"

    def get_cache_key(self, key: str, version: Optional[str] = None) -> str:
        """
        生成完整的缓存 key

        Args:
            key: 原始 key
            version: 版本号，留空则使用配置的版本

        Returns:
            完整的缓存 key，例如：infoman:v1.0.0:user:123
        """
        version = version or self.REDIS_CACHE_VERSION or "default"
        return f"{self.REDIS_CACHE_PREFIX}:v{version}:{key}"

    def to_dict(self) -> dict:
        """
        转换为字典（隐藏敏感信息）

        Returns:
            配置字典
        """
        return {
            "enabled": self.REDIS_ENABLED,
            "backend": self.REDIS_BACKEND,
            "host": self.REDIS_HOST,
            "port": self.REDIS_PORT,
            "db": self.REDIS_DB,
            "password": "***" if self.REDIS_PASSWORD else None,
            "max_connections": self.REDIS_MAX_CONNECTIONS,
            "cache_prefix": self.REDIS_CACHE_PREFIX,
            "cache_expire": self.REDIS_CACHE_EXPIRE,
            "configured": self.redis_configured,
        }


def get_redis_config() -> RedisConfig:
    return RedisConfig()


def print_redis_config():
    config = get_redis_config()

    print("=" * 60)
    print("Redis/DragonflyDB 缓存配置")
    print("=" * 60)

    for key, value in config.to_dict().items():
        print(f"  {key:20s}: {value}")

    print("=" * 60)
    print(f"  连接 URL: {config.connection_url or '未配置'}")
    print("=" * 60)


# ========== 使用示例 ==========

if __name__ == "__main__":
    # 打印配置
    print_redis_config()

    # 测试缓存 key 生成
    config = get_redis_config()

    print("\n缓存 Key 示例：")
    print(f"  user:123 -> {config.get_cache_key('user:123')}")
    print(f"  session:abc -> {config.get_cache_key('session:abc', version='2.0.0')}")



"""
日志系统配置模块
支持：本地文件 + Loki 远程推送 + 动态配置
"""
import warnings
from typing import Literal, Optional, Dict, Any
from pathlib import Path
from pydantic import Field, field_validator, model_validator, HttpUrl
from pydantic_settings import BaseSettings, SettingsConfigDict


class LogConfig(BaseSettings):
    """
    日志系统配置

    设计原则：
    1. 默认值适合开发环境
    2. 生产环境通过环境变量覆盖
    3. 自动根据 ENV 调整配置
    4. 所有配置都有合理的约束
    """
    # ========== 环境 ==========
    ENV: Literal["dev", "test", "prod"] = Field(default="dev")

    # =================================================================
    # 日志级别配置
    # =================================================================

    LOG_LEVEL: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] = Field(
        default="INFO",
        description="全局日志级别"
    )

    LOG_LEVEL_CONSOLE: Optional[Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]] = Field(
        default=None,
        description="控制台日志级别（None 则使用 LOG_LEVEL）"
    )

    LOG_LEVEL_FILE: Optional[Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]] = Field(
        default=None,
        description="文件日志级别（None 则使用 LOG_LEVEL）"
    )

    LOG_LEVEL_LOKI: Optional[Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]] = Field(
        default=None,
        description="Loki 日志级别（None 则使用 LOG_LEVEL）"
    )

    # =================================================================
    # 日志输出控制
    # =================================================================

    LOG_ENABLE_CONSOLE: bool = Field(
        default=True,
        description="启用控制台输出"
    )

    LOG_ENABLE_FILE: bool = Field(
        default=True,
        description="启用文件输出"
    )

    LOG_ENABLE_LOKI: bool = Field(
        default=False,
        description="启用 Loki 远程推送"
    )

    # =================================================================
    # 文件存储配置
    # =================================================================

    LOG_DIR: Path = Field(
        default=Path("./logs"),
        description="日志根目录"
    )

    LOG_RETENTION: str = Field(
        default="7 days",
        description="日志保留时间（格式: '7 days', '2 weeks', '1 month', '90 days'）"
    )

    LOG_ROTATION: str = Field(
        default="200 MB",
        description="日志轮转大小（格式: '100 MB', '500 MB', '1 GB'）"
    )

    LOG_COMPRESSION: Literal["zip", "gz", "bz2", "xz", None] = Field(
        default="zip",
        description="日志压缩格式（None 表示不压缩）"
    )

    LOG_FILE_ENCODING: str = Field(
        default="utf-8",
        description="日志文件编码"
    )

    # =================================================================
    # 日志格式配置
    # =================================================================

    LOG_FORMAT: Literal["simple", "detailed", "debug", "json", "pro"] = Field(
        default="detailed",
        description="日志格式预设"
    )

    LOG_ENABLE_CONSOLE_COLOR: bool = Field(
        default=True,
        description="启用控制台颜色输出"
    )

    LOG_RECORD_CALLER: bool = Field(
        default=True,
        description="记录调用者信息（文件名、函数名、行号）"
    )

    LOG_RECORD_PROCESS: bool = Field(
        default=False,
        description="记录进程信息（进程 ID、线程 ID）"
    )

    LOG_ENABLE_BACKTRACE: bool = Field(
        default=True,
        description="启用异常回溯（显示完整调用栈）"
    )

    LOG_ENABLE_DIAGNOSE: bool = Field(
        default=True,
        description="启用诊断模式（显示变量值）"
    )

    # =================================================================
    # 上下文配置
    # =================================================================

    LOG_ENABLE_CONTEXT: bool = Field(
        default=True,
        description="启用上下文管理（request_id, user_id 等）"
    )

    LOG_CONTEXT_FIELDS: list[str] = Field(
        default=["request_id", "user_id", "trace_id", "span_id"],
        description="默认上下文字段"
    )

    # =================================================================
    # 性能配置
    # =================================================================

    LOG_ENABLE_ASYNC: bool = Field(
        default=True,
        description="启用异步日志写入（提升性能）"
    )

    LOG_QUEUE_SIZE: int = Field(
        default=1000,
        ge=100,
        le=100000,
        description="异步队列大小（100-100000）"
    )

    # =================================================================
    # Loki 推送配置
    # =================================================================

    LOG_LOKI_URL: str = Field(
        default="http://loki:3100",
        description="Loki 服务地址"
    )

    LOG_LOKI_BATCH_SIZE: int = Field(
        default=100,
        ge=10,
        le=1000,
        description="批量推送大小（10-1000）"
    )

    LOG_LOKI_FLUSH_INTERVAL: float = Field(
        default=5.0,
        ge=1.0,
        le=60.0,
        description="刷新间隔（1-60 秒）"
    )

    LOG_LOKI_TIMEOUT: float = Field(
        default=10.0,
        ge=1.0,
        le=30.0,
        description="推送超时（1-30 秒）"
    )

    LOG_LOKI_RETRY_TIMES: int = Field(
        default=3,
        ge=0,
        le=10,
        description="重试次数（0-10）"
    )

    LOG_LOKI_RETRY_BACKOFF: float = Field(
        default=2.0,
        ge=1.0,
        le=10.0,
        description="重试退避系数（1.0-10.0）"
    )

    LOG_LOKI_LABELS: Dict[str, str] = Field(
        default_factory=dict,
        description="自定义 Loki Labels（会与默认 labels 合并）"
    )

    LOG_LOKI_ENABLE_FALLBACK: bool = Field(
        default=True,
        description="启用降级策略（Loki 失败时写备份文件）"
    )

    LOG_LOKI_FALLBACK_FILE: str = Field(
        default="loki_backup.log",
        description="Loki 备份文件名"
    )

    # =================================================================
    # 日志过滤配置
    # =================================================================

    LOG_FILTER_MODULES: list[str] = Field(
        default_factory=lambda: ["uvicorn.access", "uvicorn.error"],
        description="需要过滤的模块列表"
    )

    LOG_FILTER_KEYWORDS: list[str] = Field(
        default_factory=lambda: ["health_check", "heartbeat"],
        description="需要过滤的关键词列表"
    )

    LOG_ENABLE_RATE_LIMIT: bool = Field(
        default=False,
        description="启用日志速率限制（防止日志风暴）"
    )

    LOG_RATE_LIMIT_MESSAGES: int = Field(
        default=100,
        ge=10,
        le=10000,
        description="速率限制：每秒最多日志数（10-10000）"
    )

    # =================================================================
    # 敏感信息脱敏配置
    # =================================================================

    LOG_ENABLE_SANITIZE: bool = Field(
        default=True,
        description="启用敏感信息脱敏"
    )

    LOG_SANITIZE_FIELDS: list[str] = Field(
        default_factory=lambda: [
            "password", "token", "secret", "api_key", "credit_card",
            "ssn", "phone", "email", "access_token", "refresh_token"
        ],
        description="需要脱敏的字段名"
    )

    LOG_SANITIZE_PATTERN: str = Field(
        default="***REDACTED***",
        description="脱敏替换文本"
    )

    # =================================================================
    # 分级日志文件配置
    # =================================================================

    LOG_ENABLE_SPLIT_BY_LEVEL: bool = Field(
        default=True,
        description="按级别分割日志文件（info.log, error.log 等）"
    )

    LOG_ENABLE_DEBUG_FILE: bool = Field(
        default=False,
        description="启用 debug.log（仅开发环境推荐）"
    )

    LOG_ENABLE_ALL_FILE: bool = Field(
        default=True,
        description="启用 all.log（包含所有级别）"
    )

    # =================================================================
    # 监控和告警配置
    # =================================================================

    LOG_ENABLE_METRICS: bool = Field(
        default=False,
        description="启用日志指标统计（日志数量、错误率等）"
    )

    LOG_ERROR_THRESHOLD: int = Field(
        default=100,
        ge=1,
        le=10000,
        description="错误日志阈值（超过后触发告警）"
    )

    LOG_ERROR_WINDOW: int = Field(
        default=60,
        ge=10,
        le=3600,
        description="错误统计窗口（秒）"
    )

    # =================================================================
    # 验证器
    # =================================================================

    @field_validator("LOG_DIR", mode="before")
    @classmethod
    def create_log_dir(cls, v) -> Path:
        """自动创建日志目录"""
        log_dir = Path(v)
        log_dir.mkdir(parents=True, exist_ok=True)
        return log_dir

    @field_validator("LOG_LOKI_URL")
    @classmethod
    def validate_loki_url(cls, v: str) -> str:
        """验证并标准化 Loki URL"""
        if not v:
            return v

        # 补充协议
        if not v.startswith(("http://", "https://")):
            v = f"http://{v}"

        # 移除尾部斜杠
        v = v.rstrip("/")

        # 验证端口
        if ":" not in v.split("//")[1]:
            v = f"{v}:3100"  # 默认端口

        return v

    @field_validator("LOG_RETENTION")
    @classmethod
    def validate_retention(cls, v: str) -> str:
        """验证保留时间格式"""
        import re
        pattern = r'^\d+\s+(day|days|week|weeks|month|months)s?$'
        if not re.match(pattern, v, re.IGNORECASE):
            raise ValueError(
                f"LOG_RETENTION 格式错误: {v}，"
                f"正确格式: '7 days', '2 weeks', '1 month'"
            )
        return v

    @field_validator("LOG_ROTATION")
    @classmethod
    def validate_rotation(cls, v: str) -> str:
        """验证轮转大小格式"""
        import re
        pattern = r'^\d+\s+(MB|GB|KB)$'
        if not re.match(pattern, v, re.IGNORECASE):
            raise ValueError(
                f"LOG_ROTATION 格式错误: {v}，"
                f"正确格式: '100 MB', '1 GB'"
            )
        return v

    def model_post_init(self, __context):
        """初始化后处理（根据环境调整配置）"""

        # ========== 开发环境 ==========
        if self.ENV == "dev":
            self.LOG_LEVEL = "DEBUG"
            self.LOG_FORMAT = "detailed"
            self.LOG_ENABLE_CONSOLE_COLOR = True
            self.LOG_ENABLE_BACKTRACE = True
            self.LOG_ENABLE_DIAGNOSE = True
            self.LOG_ENABLE_DEBUG_FILE = True
            self.LOG_RETENTION = "3 days"
            self.LOG_ENABLE_LOKI = False

        # ========== 测试环境 ==========
        elif self.ENV == "test":
            self.LOG_LEVEL = "INFO"
            self.LOG_FORMAT = "detailed"
            self.LOG_RETENTION = "7 days"

            if not self.LOG_ENABLE_LOKI:
                warnings.warn(
                    "测试环境建议启用 Loki: LOG_ENABLE_LOKI=true",
                    UserWarning
                )

        # ========== 预发布环境 ==========
        elif self.ENV == "staging":
            self.LOG_LEVEL = "INFO"
            self.LOG_FORMAT = "pro"
            self.LOG_ENABLE_CONSOLE_COLOR = False
            self.LOG_RETENTION = "14 days"

            if not self.LOG_ENABLE_LOKI:
                raise ValueError("预发布环境必须启用 Loki: LOG_ENABLE_LOKI=true")

        # ========== 生产环境 ==========
        elif self.ENV == "prod":
            if self.LOG_LEVEL == "DEBUG":
                self.LOG_LEVEL = "INFO"

            self.LOG_FORMAT = "json"
            self.LOG_ENABLE_CONSOLE_COLOR = False
            self.LOG_ENABLE_DIAGNOSE = False
            self.LOG_ENABLE_DEBUG_FILE = False

            if self.LOG_RETENTION == "7 days":
                self.LOG_RETENTION = "30 days"

            if self.LOG_ROTATION == "200 MB":
                self.LOG_ROTATION = "500 MB"

            # if not self.LOG_ENABLE_LOKI:
            #     raise ValueError("生产环境必须启用 Loki: LOG_ENABLE_LOKI=true")

            self.LOG_ENABLE_METRICS = True
            self.LOG_ENABLE_SANITIZE = True

        # 设置分级日志级别
        if self.LOG_LEVEL_CONSOLE is None:
            self.LOG_LEVEL_CONSOLE = self.LOG_LEVEL

        if self.LOG_LEVEL_FILE is None:
            self.LOG_LEVEL_FILE = self.LOG_LEVEL

        if self.LOG_LEVEL_LOKI is None:
            self.LOG_LEVEL_LOKI = "INFO" if self.LOG_LEVEL == "DEBUG" else self.LOG_LEVEL

        # 验证依赖
        if self.LOG_ENABLE_LOKI:
            try:
                import httpx
            except ImportError:
                raise ValueError(
                    "启用 Loki 需要安装 httpx: pip install httpx"
                )

        if self.LOG_ENABLE_ASYNC and self.LOG_QUEUE_SIZE < 100:
            import warnings
            warnings.warn(
                f"异步队列大小过小 ({self.LOG_QUEUE_SIZE})，建议至少 1000",
                UserWarning
            )

    @model_validator(mode='after')
    def validate_dependencies(self) -> 'LogConfig':
        """验证依赖关系"""

        # Loki 启用时，检查 httpx
        if self.LOG_ENABLE_LOKI:
            try:
                import httpx
            except ImportError:
                raise ValueError(
                    "启用 Loki 需要安装 httpx: pip install httpx"
                )

        # 异步日志启用时，队列大小必须合理
        if self.LOG_ENABLE_ASYNC and self.LOG_QUEUE_SIZE < 100:
            import warnings
            warnings.warn(
                f"异步队列大小过小 ({self.LOG_QUEUE_SIZE})，建议至少 1000",
                UserWarning
            )

        return self

    # =================================================================
    # 辅助方法
    # =================================================================

    def get_loki_labels(self) -> Dict[str, str]:
        """获取完整的 Loki Labels"""
        import socket

        default_labels = {
            "app": self.APP_NAME,
            "env": self.ENV,
            "hostname": socket.gethostname(),
        }

        # 合并自定义 labels
        return {**default_labels, **self.LOG_LOKI_LABELS}

    def get_log_file_path(self, log_type: str) -> Path:
        """获取日志文件路径"""
        return self.LOG_DIR / f"{log_type}.log"

    def is_production(self) -> bool:
        """是否生产环境"""
        return self.ENV == "prod"

    def is_development(self) -> bool:
        """是否开发环境"""
        return self.ENV == "dev"

    def should_log_to_console(self) -> bool:
        """是否输出到控制台"""
        # 生产环境默认不输出控制台（除非明确启用）
        return self.LOG_ENABLE_CONSOLE

    # =================================================================
    # Pydantic 配置
    # =================================================================

    model_config = SettingsConfigDict(
        env_prefix="",
        case_sensitive=True,
        extra="ignore",
        validate_assignment=True,  # 赋值时验证
        frozen=False,  # 允许修改（用于动态调整）
    )


# =================================================================
# 导出单例
# =================================================================

_log_config_instance: Optional[LogConfig] = None


def get_log_config() -> LogConfig:
    """获取日志配置单例"""
    global _log_config_instance
    if _log_config_instance is None:
        _log_config_instance = LogConfig()
    return _log_config_instance


def reload_log_config() -> LogConfig:
    """重新加载日志配置"""
    global _log_config_instance
    _log_config_instance = LogConfig()
    return _log_config_instance


# =================================================================
# 配置验证工具
# =================================================================

def validate_log_config() -> tuple[bool, list[str]]:
    """
    验证日志配置

    Returns:
        (is_valid, errors)
    """
    errors = []

    try:
        config = get_log_config()

        # 检查日志目录权限
        test_file = config.LOG_DIR / ".write_test"
        try:
            test_file.touch()
            test_file.unlink()
        except Exception as e:
            errors.append(f"日志目录不可写: {config.LOG_DIR} - {e}")

        # 检查 Loki 连接
        if config.LOG_ENABLE_LOKI:
            try:
                import httpx
                response = httpx.get(
                    f"{config.LOG_LOKI_URL}/ready",
                    timeout=5.0
                )
                if response.status_code != 200:
                    errors.append(f"Loki 服务不可用: {config.LOG_LOKI_URL}")
            except Exception as e:
                errors.append(f"无法连接 Loki: {e}")

        return len(errors) == 0, errors

    except Exception as e:
        return False, [f"配置加载失败: {e}"]


# =================================================================
# 使用示例
# =================================================================

if __name__ == "__main__":
    # 测试配置加载
    config = get_log_config()

    print("=" * 60)
    print("日志配置信息")
    print("=" * 60)
    print(f"应用名称: {config.APP_NAME}")
    print(f"运行环境: {config.ENV}")
    print(f"日志级别: {config.LOG_LEVEL}")
    print(f"日志目录: {config.LOG_DIR}")
    print(f"日志格式: {config.LOG_FORMAT}")
    print(f"控制台输出: {config.LOG_ENABLE_CONSOLE}")
    print(f"文件输出: {config.LOG_ENABLE_FILE}")
    print(f"Loki 推送: {config.LOG_ENABLE_LOKI}")

    if config.LOG_ENABLE_LOKI:
        print(f"\nLoki 配置:")
        print(f"  URL: {config.LOG_LOKI_URL}")
        print(f"  批量大小: {config.LOG_LOKI_BATCH_SIZE}")
        print(f"  刷新间隔: {config.LOG_LOKI_FLUSH_INTERVAL}s")
        print(f"  Labels: {config.get_loki_labels()}")

    print(f"\n性能配置:")
    print(f"  异步日志: {config.LOG_ENABLE_ASYNC}")
    print(f"  队列大小: {config.LOG_QUEUE_SIZE}")

    print("\n" + "=" * 60)

    # 验证配置
    is_valid, errors = validate_log_config()
    if is_valid:
        print("✅ 配置验证通过")
    else:
        print("❌ 配置验证失败:")
        for error in errors:
            print(f"  - {error}")



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/7/9 16:10
# Author     ：Maxwell
# Description：
"""
from pydantic import BaseModel
from datetime import datetime


class BaseResp(BaseModel):
    class Config:
        arbitrary_types_allowed = True

    def _format_price(self, value: float) -> str:
        """格式化价格显示（带两位小数）"""
        if value:
            return f"{value:.2f} 元"
        return ""

    def _format_percentage(self, value: float) -> str:
        """格式化百分比显示（带百分号）"""
        if value:
            return f"{value:.2%}"
        return ""

    def _format_volume(self, value: int) -> str:
        """格式化成交量显示（带千位分隔符）"""
        if value:
            return f"{value:,} 股"
        return ""

    def _format_amount(self, value: float) -> str:
        """格式化成交额显示（带千位分隔符）"""
        if value:
            return f"{value:,} 元"
        return ""

    def _format_time(self, timestamp: int) -> str:
        """将时间戳转换为可读时间"""
        if timestamp:
            return datetime.fromtimestamp(timestamp / 1000).strftime(
                "%Y-%m-%d %H:%M:%S"
            )
        return ""



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/6/22 12:07
# Author     ：Maxwell
# Description：
"""
import time
import functools
from typing import Callable, TypeVar, Any, Optional, Union, Dict, List, Tuple
from inspect import iscoroutinefunction
from infoman.logger import logger


T = TypeVar("T")
F = TypeVar("F", bound=Callable[..., Any])


def safe_execute(
    func: F, default_return: Any = None, log_error: bool = True, reraise: bool = False
) -> F:
    """
    安全执行装饰器 - 捕获函数执行过程中的异常，防止程序崩溃

    参数:
        default_return: 发生异常时的默认返回值
        log_error: 是否记录错误日志
        reraise: 是否重新抛出异常

    示例:
        @safe_execute
        def my_function():
            # 可能抛出异常的代码

        @safe_execute(default_return=[], log_error=True)
        async def my_async_function():
            # 可能抛出异常的异步代码
    """
    # 处理直接使用不带参数的装饰器情况
    if callable(func) and not isinstance(func, type):

        @functools.wraps(func)
        async def async_wrapper(*args, **kwargs):
            try:
                return await func(*args, **kwargs)
            except Exception as e:
                if log_error:
                    logger.error(f"Error in {func.__name__}: {e}", exc_info=True)
                if reraise:
                    raise
                return default_return

        @functools.wraps(func)
        def sync_wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                if log_error:
                    logger.error(f"Error in {func.__name__}: {e}", exc_info=True)
                if reraise:
                    raise
                return default_return

        if iscoroutinefunction(func):
            return async_wrapper
        else:
            return sync_wrapper

    # 处理带参数的装饰器情况
    def decorator(fn: F) -> F:
        @functools.wraps(fn)
        async def async_wrapper(*args, **kwargs):
            try:
                return await fn(*args, **kwargs)
            except Exception as e:
                if log_error:
                    logger.error(f"Error in {fn.__name__}: {e}", exc_info=True)
                if reraise:
                    raise
                return default_return

        @functools.wraps(fn)
        def sync_wrapper(*args, **kwargs):
            try:
                return fn(*args, **kwargs)
            except Exception as e:
                if log_error:
                    logger.error(f"Error in {fn.__name__}: {e}", exc_info=True)
                if reraise:
                    raise
                return default_return

        if iscoroutinefunction(fn):
            return async_wrapper
        else:
            return sync_wrapper

    return decorator



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2024/1/12 10:27
# Author     ：Maxwell
# Description：
"""
import time
import traceback
from fastapi import Request, Response
from starlette.middleware.base import BaseHTTPMiddleware
from infoman.logger import logger
from infoman.utils.http.info import ClientInfoExtractor
from starlette.responses import StreamingResponse, FileResponse


class LoggingMiddleware(BaseHTTPMiddleware):

    @staticmethod
    def format_size(size_bytes: int) -> str:
        if size_bytes < 1024:
            return f"{size_bytes}B"
        elif size_bytes < 1024 * 1024:
            return f"{size_bytes / 1024:.2f}KB"
        else:
            return f"{size_bytes / (1024 * 1024):.2f}MB"

    async def dispatch(self, request: Request, call_next) -> Response:
        start_time = time.monotonic()
        client_ip = ClientInfoExtractor.client_ip(request=request)
        path = request.url.path[:200]

        try:
            response = await call_next(request)
            elapsed_ms = int((time.monotonic() - start_time) * 1000)
            # ===== 1. 检测特殊响应类型 =====
            if isinstance(response, (StreamingResponse, FileResponse)):
                content_length = response.headers.get("content-length", "unknown")
                logger.info(
                    f"Req: ip={client_ip}, elapsed_ms={elapsed_ms}, "
                    f"path={path}, status={response.status_code}, "
                    f"type=streaming, size={content_length}"
                )
                return response

            # ===== 2. 检测 Content-Length =====
            content_length = response.headers.get("content-length")
            if content_length:
                response_size = int(content_length)
                logger.info(
                    f"Req: ip={client_ip}, elapsed_ms={elapsed_ms}, "
                    f"path={path}, status={response.status_code}, "
                    f"size={self.format_size(response_size)}"
                )
                return response

            # ===== 3. 读取内容计算大小 =====
            response_body = b""
            async for chunk in response.body:
                response_body += chunk

            response_size = len(response_body)

            logger.info(
                f"Req: ip={client_ip}, elapsed_ms={elapsed_ms}, "
                f"path={path}, status={response.status_code}, "
                f"size={self.format_size(response_size)}"
            )

            return Response(
                content=response_body,
                status_code=response.status_code,
                headers=dict(response.headers),
                media_type=response.media_type,
            )

        except Exception as e:
            logger.error(
                f"Global error: ip={client_ip}, path={path}, "
                f"traceback={traceback.format_exc()}"
            )
            return Response(content="Server error.", status_code=500)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

from typing import Any, Iterable, Sequence

from .base import VectorClient, VectorPoint, VectorFilter

try:
    from qdrant_client import AsyncQdrantClient
    from qdrant_client.models import (
        Distance,
        VectorParams,
        PointStruct,
        Filter as QFilter,
        FieldCondition,
        MatchAny,
        MatchValue,
    )
except Exception:  # pragma: no cover - optional dependency at import time
    AsyncQdrantClient = None  # type: ignore
    Distance = VectorParams = PointStruct = object  # type: ignore
    QFilter = FieldCondition = MatchAny = MatchValue = object  # type: ignore


_DIST_MAP = {
    "cosine": Distance.COSINE if hasattr(Distance, "COSINE") else "cosine",
    "dot": Distance.DOT if hasattr(Distance, "DOT") else "dot",
    "euclid": Distance.EUCLID if hasattr(Distance, "EUCLID") else "euclid",
}


def _to_qfilter(f: VectorFilter | None) -> QFilter | None:
    if f is None:
        return None
    must = []
    must_not = []
    if f.must:
        for k, v in f.must.items():
            if isinstance(v, (list, tuple, set)):
                must.append(FieldCondition(key=k, match=MatchAny(any=list(v))))
            else:
                must.append(FieldCondition(key=k, match=MatchValue(value=v)))
    if f.must_not:
        for k, v in f.must_not.items():
            if isinstance(v, (list, tuple, set)):
                must_not.append(FieldCondition(key=k, match=MatchAny(any=list(v))))
            else:
                must_not.append(FieldCondition(key=k, match=MatchValue(value=v)))
    return QFilter(must=must or None, must_not=must_not or None)


class QdrantVectorClient(VectorClient):
    def __init__(self, client: Any):
        if AsyncQdrantClient is not None and not isinstance(client, AsyncQdrantClient):
            # Allow duck typing (useful in tests), but warn in docs
            pass
        self.client = client

    async def ensure_collection(
        self, name: str, vector_size: int, distance: str = "cosine", on_disk: bool = False
    ) -> None:
        dist = _DIST_MAP.get(distance, _DIST_MAP["cosine"])
        exists = await self.client.collection_exists(collection_name=name)
        if exists:
            return
        await self.client.create_collection(
            collection_name=name,
            vectors_config=VectorParams(size=vector_size, distance=dist, on_disk=on_disk),
        )

    async def upsert(self, collection: str, points: Iterable[VectorPoint], wait: bool = False) -> None:
        payloads = []
        qpoints: list[PointStruct] = []
        for p in points:
            payload = dict(p.payload) if p.payload else None
            payloads.append(payload)
            qpoints.append(PointStruct(id=p.id, vector=list(p.vector), payload=payload))
        if not qpoints:
            return
        await self.client.upsert(collection_name=collection, points=qpoints, wait=wait)

    async def delete(
        self,
        collection: str,
        ids: Iterable[str | int] | None = None,
        filt: VectorFilter | None = None,
        wait: bool = False,
    ) -> None:
        if ids:
            await self.client.delete(collection_name=collection, points=list(ids), wait=wait)
            return
        qf = _to_qfilter(filt)
        if qf is not None:
            await self.client.delete(collection_name=collection, filter=qf, wait=wait)

    async def search(
        self,
        collection: str,
        query_vector: Sequence[float],
        limit: int = 10,
        filt: VectorFilter | None = None,
        with_vectors: bool = False,
    ) -> list[dict[str, Any]]:
        qf = _to_qfilter(filt)
        res = await self.client.search(
            collection_name=collection,
            query_vector=list(query_vector),
            limit=limit,
            filter=qf,
            with_vectors=with_vectors,
        )
        results: list[dict[str, Any]] = []
        for r in res:
            results.append(
                {
                    "id": r.id,
                    "score": r.score,
                    "payload": getattr(r, "payload", None),
                    "vector": getattr(r, "vector", None) if with_vectors else None,
                }
            )
        return results




# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/6/22 12:10
# Author     ：Maxwell
# Description：
"""
import time
import asyncio
import functools
from typing import Callable, TypeVar, Any, Optional, Union, Dict, List, Tuple
from infoman.logger import logger
from inspect import iscoroutinefunction

T = TypeVar("T")
F = TypeVar("F", bound=Callable[..., Any])


def retry(
    max_attempts: int = 3,
    delay_seconds: float = 1.0,
    backoff_factor: float = 2.0,
    exceptions: Tuple[Exception, ...] = (Exception,),
    logger_name: Optional[str] = None,
) -> Callable[[F], F]:
    """
    重试装饰器 - 在发生特定异常时自动重试函数

    参数:
        max_attempts: 最大尝试次数
        delay_seconds: 初始延迟时间(秒)
        backoff_factor: 退避因子(每次重试后延迟时间的乘数)
        exceptions: 触发重试的异常类型
        logger_name: 自定义日志记录器名称

    示例:
        @retry(max_attempts=5, exceptions=(ConnectionError, TimeoutError))
        async def connect_to_service():
            # 连接服务的代码
    """

    def decorator(func: F) -> F:
        @functools.wraps(func)
        async def async_wrapper(*args, **kwargs):
            attempt = 1
            current_delay = delay_seconds

            while True:
                try:
                    return await func(*args, **kwargs)
                except exceptions as e:
                    if attempt >= max_attempts:
                        logger.error(
                            f"Function {func.__name__} failed after {max_attempts} attempts. "
                            f"Last error: {str(e)}"
                        )
                        raise

                    logger.warning(
                        f"Attempt {attempt}/{max_attempts} for {func.__name__} failed: {str(e)}. "
                        f"Retrying in {current_delay:.2f}s"
                    )

                    await asyncio.sleep(current_delay)
                    attempt += 1
                    current_delay *= backoff_factor

        @functools.wraps(func)
        def sync_wrapper(*args, **kwargs):
            attempt = 1
            current_delay = delay_seconds

            while True:
                try:
                    return func(*args, **kwargs)
                except exceptions as e:
                    if attempt >= max_attempts:
                        logger.error(
                            f"Function {func.__name__} failed after {max_attempts} attempts. "
                            f"Last error: {str(e)}"
                        )
                        raise

                    logger.warning(
                        f"Attempt {attempt}/{max_attempts} for {func.__name__} failed: {str(e)}. "
                        f"Retrying in {current_delay:.2f}s"
                    )

                    time.sleep(current_delay)
                    attempt += 1
                    current_delay *= backoff_factor

        if iscoroutinefunction(func):
            return async_wrapper
        else:
            return sync_wrapper

    return decorator



from enum import Enum
from dataclasses import dataclass
from typing import Optional, Dict, Any
from http import HTTPStatus


@dataclass
class ErrorProperties:
    code: int
    message: str
    message_en: str
    status_code: int = HTTPStatus.INTERNAL_SERVER_ERROR
    details: Optional[Dict[str, Any]] = None


class AppError(Enum):

    @property
    def code(self) -> int:
        return self.value.code

    @property
    def message(self) -> str:
        return self.value.message

    @property
    def message_en(self) -> str:
        return self.value.message_en

    @property
    def status_code(self) -> int:
        return self.value.status_code

    @property
    def details(self) -> Optional[Dict[str, Any]]:
        return self.value.details

    def with_details(self, details: Dict[str, Any]) -> ErrorProperties:
        return ErrorProperties(
            code=self.code,
            message=self.message,
            message_en=self.message_en,
            status_code=self.status_code,
            details=details,
        )


class AppSystemError(AppError):
    UNKNOWN_ERROR = ErrorProperties(
        100000, "系统未知异常", "Unknown system error", HTTPStatus.INTERNAL_SERVER_ERROR
    )

    SYSTEM_ERROR = ErrorProperties(
        100001,
        "系统内部错误",
        "Internal server error",
        HTTPStatus.INTERNAL_SERVER_ERROR,
    )

    TIMEOUT_ERROR = ErrorProperties(
        100002, "系统处理超时", "System processing timeout", HTTPStatus.GATEWAY_TIMEOUT
    )

    SERVICE_UNAVAILABLE = ErrorProperties(
        100003,
        "服务暂时不可用",
        "Service temporarily unavailable",
        HTTPStatus.SERVICE_UNAVAILABLE,
    )

    RESOURCE_EXHAUSTED = ErrorProperties(
        100004,
        "系统资源不足",
        "System resources exhausted",
        HTTPStatus.TOO_MANY_REQUESTS,
    )

    CONFIGURATION_ERROR = ErrorProperties(
        100005,
        "系统配置错误",
        "System configuration error",
        HTTPStatus.INTERNAL_SERVER_ERROR,
    )


class RequestError(AppError):
    VALIDATION_ERROR = ErrorProperties(
        200000,
        "请求数据验证失败",
        "Request data validation failed",
        HTTPStatus.BAD_REQUEST,
    )

    INVALID_PARAMETER = ErrorProperties(
        200001, "无效的请求参数", "Invalid request parameter", HTTPStatus.BAD_REQUEST
    )

    MISSING_PARAMETER = ErrorProperties(
        200002, "缺少必要参数", "Missing required parameter", HTTPStatus.BAD_REQUEST
    )

    INVALID_FORMAT = ErrorProperties(
        200003, "请求格式错误", "Invalid request format", HTTPStatus.BAD_REQUEST
    )

    REQUEST_TOO_LARGE = ErrorProperties(
        200004,
        "请求体过大",
        "Request body too large",
        HTTPStatus.REQUEST_ENTITY_TOO_LARGE,
    )

    RATE_LIMITED = ErrorProperties(
        200005, "请求频率超限", "Request rate limited", HTTPStatus.TOO_MANY_REQUESTS
    )

    UNSUPPORTED_MEDIA_TYPE = ErrorProperties(
        200006,
        "不支持的媒体类型",
        "Unsupported media type",
        HTTPStatus.UNSUPPORTED_MEDIA_TYPE,
    )


class DatabaseError(AppError):
    VALIDATION_ERROR = ErrorProperties(
        300000, "数据验证失败", "Data validation failed", HTTPStatus.BAD_REQUEST
    )

    INTEGRITY_ERROR = ErrorProperties(
        300001,
        "数据完整性约束冲突",
        "Data integrity constraint violation",
        HTTPStatus.CONFLICT,
    )

    RECORD_NOT_FOUND = ErrorProperties(
        300002, "记录不存在", "Record not found", HTTPStatus.NOT_FOUND
    )

    DUPLICATE_ENTRY = ErrorProperties(
        300003, "记录已存在", "Duplicate record", HTTPStatus.CONFLICT
    )

    OPERATIONAL_ERROR = ErrorProperties(
        300004,
        "数据库操作错误",
        "Database operational error",
        HTTPStatus.INTERNAL_SERVER_ERROR,
    )

    CONNECTION_ERROR = ErrorProperties(
        300005,
        "数据库连接错误",
        "Database connection error",
        HTTPStatus.SERVICE_UNAVAILABLE,
    )

    TRANSACTION_ERROR = ErrorProperties(
        300006,
        "事务处理错误",
        "Transaction processing error",
        HTTPStatus.INTERNAL_SERVER_ERROR,
    )

    QUERY_TIMEOUT = ErrorProperties(
        300007, "查询超时", "Query timeout", HTTPStatus.GATEWAY_TIMEOUT
    )


class BusinessError(AppError):

    INVALID_OPERATION = ErrorProperties(
        400000, "无效的操作", "Invalid operation", HTTPStatus.BAD_REQUEST
    )

    STATE_CONFLICT = ErrorProperties(
        400001, "状态冲突", "State conflict", HTTPStatus.CONFLICT
    )

    RESOURCE_LOCKED = ErrorProperties(
        400002, "资源已锁定", "Resource locked", HTTPStatus.LOCKED
    )

    QUOTA_EXCEEDED = ErrorProperties(
        400003, "配额超限", "Quota exceeded", HTTPStatus.FORBIDDEN
    )

    PAYMENT_REQUIRED = ErrorProperties(
        400004, "需要付款", "Payment required", HTTPStatus.PAYMENT_REQUIRED
    )

    BUSINESS_RULE_VIOLATION = ErrorProperties(
        400005, "违反业务规则", "Business rule violation", HTTPStatus.BAD_REQUEST
    )


class SecurityError(AppError):
    """
    安全相关错误

    包括认证、授权、访问控制等错误
    错误代码范围：500000-599999
    """

    UNAUTHORIZED = ErrorProperties(
        500000, "未授权访问", "Unauthorized access", HTTPStatus.UNAUTHORIZED
    )

    FORBIDDEN = ErrorProperties(
        500001, "禁止访问", "Access forbidden", HTTPStatus.FORBIDDEN
    )

    INVALID_CREDENTIALS = ErrorProperties(
        500002, "无效的凭证", "Invalid credentials", HTTPStatus.UNAUTHORIZED
    )

    TOKEN_EXPIRED = ErrorProperties(
        500003, "令牌已过期", "Token expired", HTTPStatus.UNAUTHORIZED
    )

    INVALID_TOKEN = ErrorProperties(
        500004, "无效的令牌", "Invalid token", HTTPStatus.UNAUTHORIZED
    )

    PERMISSION_DENIED = ErrorProperties(
        500005, "权限不足", "Permission denied", HTTPStatus.FORBIDDEN
    )

    ACCOUNT_LOCKED = ErrorProperties(
        500006, "账户已锁定", "Account locked", HTTPStatus.FORBIDDEN
    )

    IP_RESTRICTED = ErrorProperties(
        500007, "IP地址受限", "IP address restricted", HTTPStatus.FORBIDDEN
    )


class ExternalServiceError(AppError):
    """
    外部服务错误

    包括第三方API、外部依赖服务等错误
    错误代码范围：600000-699999
    """

    SERVICE_UNAVAILABLE = ErrorProperties(
        600000, "外部服务不可用", "External service unavailable", HTTPStatus.BAD_GATEWAY
    )

    REQUEST_FAILED = ErrorProperties(
        600001,
        "外部服务请求失败",
        "External service request failed",
        HTTPStatus.BAD_GATEWAY,
    )

    RESPONSE_ERROR = ErrorProperties(
        600002,
        "外部服务响应错误",
        "External service response error",
        HTTPStatus.BAD_GATEWAY,
    )

    TIMEOUT = ErrorProperties(
        600003, "外部服务超时", "External service timeout", HTTPStatus.GATEWAY_TIMEOUT
    )

    INVALID_RESPONSE = ErrorProperties(
        600004,
        "外部服务返回无效数据",
        "Invalid data returned from external service",
        HTTPStatus.BAD_GATEWAY,
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

from typing import Any, Sequence

from fastapi import FastAPI

from .base import VectorPoint, VectorFilter
from . import VectorClient
from infoman.service.models.type.embed import (
    EmbedModel,
    EmbedCollection,
    EmbedDataType,
)


class VectorService:
    """High-level vector operations bound to Embed* config.

    This provides convenience methods using `EmbedModel` and `EmbedDataType`
    to compute collection names and dimensions.
    """

    def __init__(self, vector_client: VectorClient):
        self.vc = vector_client

    @staticmethod
    def collection_name(data_type: EmbedDataType, model: EmbedModel) -> str:
        return EmbedCollection.collection_name(data_type, model)

    async def ensure_collection(self, data_type: EmbedDataType, model: EmbedModel) -> str:
        name = self.collection_name(data_type, model)
        await self.vc.ensure_collection(name, model.value.dem, distance="cosine")
        return name

    async def upsert(
        self,
        data_type: EmbedDataType,
        model: EmbedModel,
        points: list[VectorPoint],
        wait: bool = False,
    ) -> str:
        name = self.collection_name(data_type, model)
        await self.vc.upsert(name, points, wait=wait)
        return name

    async def search(
        self,
        data_type: EmbedDataType,
        model: EmbedModel,
        query_vector: Sequence[float],
        limit: int = 10,
        filt: VectorFilter | None = None,
        with_vectors: bool = False,
    ) -> list[dict[str, Any]]:
        name = self.collection_name(data_type, model)
        return await self.vc.search(name, query_vector, limit, filt, with_vectors)


def get_vector_service(app: FastAPI) -> VectorService | None:
    vc = getattr(app.state, "vector_client", None)
    if vc is None:
        return None
    return VectorService(vc)




# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/6/22 12:02
# Author     ：Maxwell
# Description：
"""

from .settings import settings
from .base import BaseConfig
from .db_relation import DatabaseConfig, DatabaseInstanceConfig
from .db_cache import RedisConfig
from .db_vector import VectorDBConfig
from .mq import MessageQueueConfig

__all__ = [
    "settings",
    "BaseConfig",
    "DatabaseConfig",
    "DatabaseInstanceConfig",
    "RedisConfig",
    "VectorDBConfig",
    "MessageQueueConfig",
]



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
向量数据库基础设施

支持:
- Qdrant: 高性能向量搜索引擎
- Milvus: 待实现

Usage:
    # 方式 1: 通过 lifespan 自动初始化
    from infoman.service.app import application
    # VectorDBManager 会在应用启动时自动初始化

    # 方式 2: 手动使用
    from infoman.service.infrastructure.db_vector import VectorDBManager

    manager = VectorDBManager()
    await manager.startup()

    # 使用 Qdrant
    client = manager.qdrant.client
    await client.create_collection(...)

    # 便捷方法
    await manager.create_collection("my_collection", vector_size=768)
    results = await manager.search("my_collection", query_vector=[...])

    # 关闭
    await manager.shutdown()
"""

from .manager import VectorDBManager
from .qdrant import QdrantBackend

__all__ = [
    "VectorDBManager",
    "QdrantBackend",
]



import asyncio
import json
from nats.aio.client import Client as NATS
from nats.aio.errors import ErrConnectionClosed, ErrTimeout, ErrNoServers
from infoman.logger import logger


class NATSClient:

    def __init__(self, servers=None, name="nats-client"):
        if servers is None:
            servers = ["nats://127.0.0.1:4222"]
        self.nc = NATS()
        self.servers = servers
        self.name = name
        self.connected = False

    async def connect(self):
        if not self.connected:
            await self.nc.connect(servers=self.servers, name=self.name)
            self.connected = True
            logger.info(f"[NATS] Connected to {self.servers}")

    async def publish(self, subject: str, message: dict):
        if not self.connected:
            await self.connect()
        try:
            payload = json.dumps(message).encode()
            await self.nc.publish(subject, payload)
            logger.info(f"[NATS] Published to {subject}: {message}")
        except Exception as e:
            logger.info(f"[NATS] Publish error: {e}")

    async def request(self, subject: str, message: dict, timeout=1.0):
        if not self.connected:
            await self.connect()
        try:
            payload = json.dumps(message).encode()
            msg = await self.nc.request(subject, payload, timeout=timeout)
            return json.loads(msg.data.decode())
        except ErrTimeout:
            logger.info("[NATS] Request timeout")
            return None
        except Exception as e:
            logger.info(f"[NATS] Request error: {e}")
            return None

    async def subscribe(self, subject: str, callback, queue: str):
        if not self.connected:
            await self.connect()
        await self.nc.subscribe(subject, cb=callback, queue=queue)

    async def close(self):
        if self.connected:
            await self.nc.drain()
            self.connected = False
            logger.info("[NATS] Connection closed")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Iterable, Mapping, Protocol, Sequence


@dataclass
class VectorPoint:
    id: str | int
    vector: Sequence[float]
    payload: Mapping[str, Any] | None = None


@dataclass
class VectorFilter:
    # Generic equality filter; concrete impl can translate
    must: dict[str, Any] | None = None
    must_not: dict[str, Any] | None = None


class VectorClient(Protocol):
    async def ensure_collection(
        self,
        name: str,
        vector_size: int,
        distance: str = "cosine",
        on_disk: bool = False,
    ) -> None: ...

    async def upsert(
        self,
        collection: str,
        points: Iterable[VectorPoint],
        wait: bool = False,
    ) -> None: ...

    async def delete(
        self,
        collection: str,
        ids: Iterable[str | int] | None = None,
        filt: VectorFilter | None = None,
        wait: bool = False,
    ) -> None: ...

    async def search(
        self,
        collection: str,
        query_vector: Sequence[float],
        limit: int = 10,
        filt: VectorFilter | None = None,
        with_vectors: bool = False,
    ) -> list[dict[str, Any]]: ...




# service/core/lifespan.py
"""
应用生命周期管理（精简版）

职责：
- 协调各个服务的启动和关闭
- 不包含具体的连接逻辑
"""

from contextlib import asynccontextmanager
from fastapi import FastAPI
from loguru import logger

from infoman.config import settings

# 导入各服务的管理器
from infoman.service.infrastructure.db_relation.manager import DatabaseManager
from infoman.service.infrastructure.db_cache.manager import RedisManager
from infoman.service.infrastructure.db_vector.manager import VectorDBManager
from infoman.service.infrastructure.mq import NATSManager


@asynccontextmanager
async def lifespan(app: FastAPI):
    """应用生命周期管理"""

    # ========== 启动 ==========
    logger.info(f"🚀 应用启动中 [{settings.APP_NAME} v{settings.APP_VERSION}]")
    logger.info(f"   环境: {settings.ENV}")

    # 初始化管理器
    db_manager = DatabaseManager()
    redis_manager = RedisManager()
    vector_manager = VectorDBManager()
    nats_manager = NATSManager()

    # 保存到 app.state
    app.state.db_manager = db_manager
    app.state.redis_manager = redis_manager
    app.state.vector_manager = vector_manager
    app.state.nats_manager = nats_manager

    try:
        # 1. 数据库
        await db_manager.startup(app)

        # 2. Redis
        await redis_manager.startup(app)

        # 3. 向量数据库
        await vector_manager.startup(app)

        # 4. 消息队列
        await nats_manager.startup(app)

        logger.success("✅ 所有服务启动完成")

    except Exception as e:
        logger.error(f"❌ 服务启动失败: {e}")
        raise

    # ========== 运行 ==========
    yield

    # ========== 关闭 ==========
    logger.info("⏹️ 应用关闭中...")

    try:
        # 按相反顺序关闭
        await nats_manager.shutdown()
        await vector_manager.shutdown()
        await redis_manager.shutdown()
        await db_manager.shutdown()

        logger.success("✅ 所有服务已关闭")

    except Exception as e:
        logger.error(f"❌ 服务关闭失败: {e}")



#!/usr/bin/env python
# -*-coding:utf-8 -*-
"""
数据库模型基类 - 支持多 ORM 后端

设计原则：
1. 用户代码只依赖抽象基类
2. 运行时根据配置选择后端
3. 100% 向前兼容现有 Tortoise 代码

Version: 0.3.0
Author: Maxwell
"""

from abc import ABC, abstractmethod
from typing import TypeVar, Generic, Optional, List, Any
from datetime import datetime

# ==================== 抽象接口层 ====================


class BaseTimestampMixin(ABC):
    """时间戳混入抽象类 - ORM 无关"""
    id: int
    created_at: datetime
    updated_at: datetime


T = TypeVar('T', bound=BaseTimestampMixin)


class BaseRepository(ABC, Generic[T]):
    """
    仓储模式抽象类 - 隔离 ORM 实现细节

    使用示例：
        >>> repo = create_repository(User)
        >>> user = await repo.get(1)
        >>> users = await repo.filter(name="Alice")
    """

    @abstractmethod
    async def get(self, id: int) -> Optional[T]:
        """根据 ID 获取单条记录"""
        pass

    @abstractmethod
    async def filter(self, **kwargs) -> List[T]:
        """根据条件筛选"""
        pass

    @abstractmethod
    async def create(self, **kwargs) -> T:
        """创建记录"""
        pass

    @abstractmethod
    async def update(self, id: int, **kwargs) -> T:
        """更新记录"""
        pass

    @abstractmethod
    async def delete(self, id: int) -> bool:
        """删除记录"""
        pass

    @abstractmethod
    async def all(self) -> List[T]:
        """获取所有记录"""
        pass

    @abstractmethod
    async def count(self, **kwargs) -> int:
        """计数"""
        pass


# ==================== Tortoise ORM 实现（向前兼容）====================

try:
    from tortoise import fields
    from tortoise.models import Model as TortoiseModel

    class TimestampMixin(TortoiseModel, BaseTimestampMixin):
        """
        Tortoise ORM 时间戳混入 - 保持现有 API 不变

        使用示例（向前兼容）：
            >>> class User(TimestampMixin):
            ...     name = fields.CharField(max_length=100)
            >>> user = await User.create(name="Alice")
        """

        class Meta:
            abstract = True

        id = fields.IntField(pk=True, null=False)
        created_at = fields.DatetimeField(auto_now_add=True, description="创建时间")
        updated_at = fields.DatetimeField(auto_now=True, description="更新时间")


    class MySQLModel(TortoiseModel):
        """MySQL 模型基类（Tortoise）"""
        class Meta:
            abstract = True
            app = 'mysql_models'


    class PostgreSQLModel(TortoiseModel):
        """PostgreSQL 模型基类（Tortoise）"""
        class Meta:
            abstract = True
            app = 'postgres_models'


    class SQLiteModel(TortoiseModel):
        """SQLite 模型基类（Tortoise）"""
        class Meta:
            abstract = True
            app = 'sqlite_models'


    class TortoiseRepository(BaseRepository[T]):
        """
        Tortoise ORM 仓储实现

        使用示例：
            >>> repo = TortoiseRepository(User)
            >>> user = await repo.create(name="Alice")
        """

        def __init__(self, model_class):
            self.model = model_class

        async def get(self, id: int) -> Optional[T]:
            try:
                return await self.model.get(id=id)
            except Exception:
                return None

        async def filter(self, **kwargs) -> List[T]:
            return await self.model.filter(**kwargs).all()

        async def create(self, **kwargs) -> T:
            return await self.model.create(**kwargs)

        async def update(self, id: int, **kwargs) -> T:
            instance = await self.model.get(id=id)
            await instance.update_from_dict(kwargs).save()
            return instance

        async def delete(self, id: int) -> bool:
            instance = await self.model.get(id=id)
            await instance.delete()
            return True

        async def all(self) -> List[T]:
            return await self.model.all()

        async def count(self, **kwargs) -> int:
            return await self.model.filter(**kwargs).count()

    _TORTOISE_AVAILABLE = True

except ImportError:
    # Tortoise 未安装，使用占位符
    _TORTOISE_AVAILABLE = False

    class TimestampMixin(BaseTimestampMixin):  # type: ignore
        """占位符：需要安装 tortoise-orm"""
        pass

    class MySQLModel:  # type: ignore
        """占位符：需要安装 tortoise-orm"""
        pass

    class PostgreSQLModel:  # type: ignore
        """占位符：需要安装 tortoise-orm"""
        pass

    class SQLiteModel:  # type: ignore
        """占位符：需要安装 tortoise-orm"""
        pass

    class TortoiseRepository(BaseRepository[T]):  # type: ignore
        """占位符：需要安装 tortoise-orm"""
        def __init__(self, model_class):
            raise ImportError("需要安装 tortoise-orm: pip install infomankit[database]")

        async def get(self, id: int) -> Optional[T]:
            raise NotImplementedError
        async def filter(self, **kwargs) -> List[T]:
            raise NotImplementedError
        async def create(self, **kwargs) -> T:
            raise NotImplementedError
        async def update(self, id: int, **kwargs) -> T:
            raise NotImplementedError
        async def delete(self, id: int) -> bool:
            raise NotImplementedError
        async def all(self) -> List[T]:
            raise NotImplementedError
        async def count(self, **kwargs) -> int:
            raise NotImplementedError


# ==================== SQLAlchemy 实现（新增）====================

try:
    from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column
    from sqlalchemy import Integer, DateTime, func, select, delete as sql_delete
    from sqlalchemy.ext.asyncio import AsyncSession

    class AlchemyBase(DeclarativeBase):
        """
        SQLAlchemy 基类

        使用示例：
            >>> class User(AlchemyBase, AlchemyTimestampMixin):
            ...     __tablename__ = "users"
            ...     name: Mapped[str] = mapped_column(String(100))
        """
        pass


    class AlchemyTimestampMixin(BaseTimestampMixin):
        """
        SQLAlchemy 时间戳混入

        提供自动管理的 id, created_at, updated_at 字段
        """
        id: Mapped[int] = mapped_column(Integer, primary_key=True)
        created_at: Mapped[datetime] = mapped_column(
            DateTime,
            server_default=func.now(),
            comment="创建时间"
        )
        updated_at: Mapped[datetime] = mapped_column(
            DateTime,
            server_default=func.now(),
            onupdate=func.now(),
            comment="更新时间"
        )


    class SQLAlchemyRepository(BaseRepository[T]):
        """
        SQLAlchemy 仓储实现

        使用示例：
            >>> session_maker = get_session_maker()
            >>> repo = SQLAlchemyRepository(User, session_maker)
            >>> user = await repo.create(name="Alice")
        """

        def __init__(self, model_class, session_factory):
            self.model = model_class
            self.session_factory = session_factory

        async def get(self, id: int) -> Optional[T]:
            async with self.session_factory() as session:
                stmt = select(self.model).where(self.model.id == id)
                result = await session.execute(stmt)
                return result.scalar_one_or_none()

        async def filter(self, **kwargs) -> List[T]:
            async with self.session_factory() as session:
                stmt = select(self.model).filter_by(**kwargs)
                result = await session.execute(stmt)
                return list(result.scalars().all())

        async def create(self, **kwargs) -> T:
            async with self.session_factory() as session:
                instance = self.model(**kwargs)
                session.add(instance)
                await session.commit()
                await session.refresh(instance)
                return instance

        async def update(self, id: int, **kwargs) -> T:
            async with self.session_factory() as session:
                stmt = select(self.model).where(self.model.id == id)
                result = await session.execute(stmt)
                instance = result.scalar_one()

                for key, value in kwargs.items():
                    setattr(instance, key, value)

                await session.commit()
                await session.refresh(instance)
                return instance

        async def delete(self, id: int) -> bool:
            async with self.session_factory() as session:
                stmt = sql_delete(self.model).where(self.model.id == id)
                await session.execute(stmt)
                await session.commit()
                return True

        async def all(self) -> List[T]:
            async with self.session_factory() as session:
                stmt = select(self.model)
                result = await session.execute(stmt)
                return list(result.scalars().all())

        async def count(self, **kwargs) -> int:
            async with self.session_factory() as session:
                from sqlalchemy import func as sql_func
                stmt = select(sql_func.count()).select_from(self.model).filter_by(**kwargs)
                result = await session.execute(stmt)
                return result.scalar()

    _SQLALCHEMY_AVAILABLE = True

except ImportError:
    # SQLAlchemy 未安装
    _SQLALCHEMY_AVAILABLE = False

    class AlchemyBase:  # type: ignore
        """占位符：需要安装 sqlalchemy"""
        pass

    class AlchemyTimestampMixin(BaseTimestampMixin):  # type: ignore
        """占位符：需要安装 sqlalchemy"""
        pass

    class SQLAlchemyRepository(BaseRepository[T]):  # type: ignore
        """占位符：需要安装 sqlalchemy"""
        def __init__(self, model_class, session_factory):
            raise ImportError("需要安装 sqlalchemy: pip install infomankit[database-alchemy]")

        async def get(self, id: int) -> Optional[T]:
            raise NotImplementedError
        async def filter(self, **kwargs) -> List[T]:
            raise NotImplementedError
        async def create(self, **kwargs) -> T:
            raise NotImplementedError
        async def update(self, id: int, **kwargs) -> T:
            raise NotImplementedError
        async def delete(self, id: int) -> bool:
            raise NotImplementedError
        async def all(self) -> List[T]:
            raise NotImplementedError
        async def count(self, **kwargs) -> int:
            raise NotImplementedError


# ==================== 工厂函数（自动选择后端）====================

def create_repository(model_class, backend: str = "auto", session_factory=None) -> BaseRepository:
    """
    创建仓储实例 - 自动选择后端

    Args:
        model_class: 模型类
        backend: 'tortoise', 'sqlalchemy', 'auto'（默认）
        session_factory: SQLAlchemy session factory（仅 SQLAlchemy 后端需要）

    Returns:
        BaseRepository 实例

    Raises:
        ValueError: 不支持的后端或无法自动检测
        ImportError: 所需的 ORM 库未安装

    Example:
        >>> # 自动检测
        >>> user_repo = create_repository(User)
        >>> users = await user_repo.all()

        >>> # 指定后端
        >>> user_repo = create_repository(User, backend="tortoise")
        >>> user = await user_repo.create(name="Alice")
    """
    if backend == "auto":
        # 自动检测模型类型
        if hasattr(model_class, '_meta') and hasattr(model_class._meta, 'table'):
            backend = "tortoise"
        elif hasattr(model_class, '__table__'):
            backend = "sqlalchemy"
        else:
            raise ValueError(
                f"无法自动检测模型类型: {model_class}. "
                f"请显式指定 backend='tortoise' 或 'sqlalchemy'"
            )

    if backend == "tortoise":
        if not _TORTOISE_AVAILABLE:
            raise ImportError(
                "Tortoise ORM 未安装。请运行: pip install infomankit[database]"
            )
        return TortoiseRepository(model_class)

    elif backend == "sqlalchemy":
        if not _SQLALCHEMY_AVAILABLE:
            raise ImportError(
                "SQLAlchemy 未安装。请运行: pip install infomankit[database-alchemy]"
            )

        if session_factory is None:
            # 尝试从全局管理器获取
            try:
                from infoman.service.infrastructure.db_relation.manager import hybrid_db_manager
                session_factory = hybrid_db_manager.get_sqlalchemy_session_maker()
            except Exception:
                raise ValueError(
                    "SQLAlchemy 后端需要提供 session_factory 参数，"
                    "或确保 HybridDatabaseManager 已初始化"
                )

        return SQLAlchemyRepository(model_class, session_factory)

    else:
        raise ValueError(f"不支持的后端: {backend}. 仅支持 'tortoise', 'sqlalchemy', 'auto'")


# ==================== 向前兼容导出 ====================

__all__ = [
    # 抽象基类
    'BaseTimestampMixin',
    'BaseRepository',

    # Tortoise（向前兼容）
    'TimestampMixin',
    'MySQLModel',
    'PostgreSQLModel',
    'SQLiteModel',
    'TortoiseRepository',

    # SQLAlchemy（新增）
    'AlchemyBase',
    'AlchemyTimestampMixin',
    'SQLAlchemyRepository',

    # 工厂函数
    'create_repository',

    # 可用性标志
    '_TORTOISE_AVAILABLE',
    '_SQLALCHEMY_AVAILABLE',
]



from infoman.utils.log import logger
from functools import wraps
from typing import (
    Any,
    Callable,
    Optional,
    Type,
    TypeVar,
    get_type_hints,
    get_origin,
    get_args,
)
import json
from pydantic import BaseModel
import inspect


T = TypeVar("T")


def redis_cache(
    prefix: str,
    ttl: int = 3600,
    model: Optional[Type[BaseModel]] = None,  # ✅ 新增：指定返回模型
    auto_detect: bool = True,  # ✅ 新增：自动检测返回类型
):
    """
    Redis 缓存装饰器（支持 Pydantic 模型反序列化）

    Args:
        prefix: 缓存键前缀
        ttl: 过期时间（秒）
        model: 返回的 Pydantic 模型类（可选，如果指定则强制使用）
        auto_detect: 是否自动从函数签名检测返回类型

    Examples:
        # 方式 1：手动指定模型
        @redis_cache(prefix="config", ttl=3600, model=ConfigKeyLogSchema)
        async def get_config(pub_key: str):
            ...

        # 方式 2：自动检测（推荐）
        @redis_cache(prefix="config", ttl=3600)
        async def get_config(pub_key: str) -> ConfigKeyLogSchema:
            ...

        # 方式 3：列表类型
        @redis_cache(prefix="configs", ttl=3600)
        async def list_configs() -> list[ConfigKeyLogSchema]:
            ...
    """

    def decorator(func: Callable) -> Callable:
        # ✅ 自动检测返回类型
        return_type = model
        is_list = False

        if auto_detect and not return_type:
            type_hints = get_type_hints(func)
            if "return" in type_hints:
                hint = type_hints["return"]

                # 处理 Optional[Model]
                origin = get_origin(hint)
                if origin is Optional or str(origin) == "typing.Union":
                    args = get_args(hint)
                    for arg in args:
                        if arg is not type(None) and (
                            isinstance(arg, type) and issubclass(arg, BaseModel)
                        ):
                            return_type = arg
                            break
                elif origin is list:
                    args = get_args(hint)
                    if (
                        args
                        and isinstance(args[0], type)
                        and issubclass(args[0], BaseModel)
                    ):
                        return_type = args[0]
                        is_list = True

                # 直接是 Model
                elif isinstance(hint, type) and issubclass(hint, BaseModel):
                    return_type = hint

        @wraps(func)
        async def wrapper(*args, **kwargs) -> Any:
            # 获取 Redis 客户端
            redis_client = _get_redis_client(args, kwargs)

            if not redis_client:
                logger.warning("Redis client not found, executing without cache")
                return await func(*args, **kwargs)

            # 构建缓存键
            cache_key = f"{prefix}:{_build_cache_key(func, args, kwargs)}"

            # ==================== 读取缓存 ====================
            try:
                cached = await redis_client.get(cache_key)
                if cached:
                    cached_data = json.loads(cached)
                    logger.debug(f"Cache HIT: {cache_key}")

                    if return_type:
                        if is_list:
                            return [return_type(**item) for item in cached_data]
                        else:
                            return return_type(**cached_data)
                    return cached_data

            except Exception as e:
                logger.error(f"Cache read error for {cache_key}: {e}")

            # ==================== 执行函数 ====================
            result = await func(*args, **kwargs)

            if result is None:
                return None

            # ==================== 写入缓存 ====================
            try:
                cache_data = _serialize_result(result)
                json_str = json.dumps(cache_data, default=str, ensure_ascii=False)
                await redis_client.setex(cache_key, ttl, json_str)
                logger.debug(f"Cache SET: {cache_key}")

            except Exception as e:
                logger.error(f"Cache write error for {cache_key}: {e}")

            return result

        return wrapper

    return decorator


def _serialize_result(result: Any) -> Any:
    """序列化结果"""
    if isinstance(result, BaseModel):
        return result.model_dump()
    elif isinstance(result, list):
        return [
            item.model_dump() if isinstance(item, BaseModel) else item
            for item in result
        ]
    elif isinstance(result, dict):
        return {
            k: v.model_dump() if isinstance(v, BaseModel) else v
            for k, v in result.items()
        }
    return result


def _get_redis_client(args: tuple, kwargs: dict):

    for arg in list(args) + list(kwargs.values()):
        if hasattr(arg, "app") and hasattr(arg.app, "state"):
            redis = getattr(arg.app.state, "redis_client", None)
            if redis:
                return redis

    return None


def _build_cache_key(func: Callable, args: tuple, kwargs: dict) -> str:
    """构建缓存键"""
    key_parts = [func.__name__]

    # 获取函数签名
    sig = inspect.signature(func)
    param_names = list(sig.parameters.keys())

    # 处理位置参数（跳过 self 和 request）
    for i, arg in enumerate(args):
        if i == 0 and param_names and param_names[0] in ["self", "cls"]:
            continue
        if _is_cacheable_value(arg):
            key_parts.append(str(arg))

    # 处理关键字参数
    for k, v in sorted(kwargs.items()):
        if k not in ["req", "request"] and _is_cacheable_value(v):
            key_parts.append(f"{k}={v}")

    return ":".join(key_parts)


def _is_cacheable_value(value: Any) -> bool:
    """判断值是否可用于缓存键"""
    return isinstance(value, (str, int, float, bool))



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/12/23 17:58
# Author     ：Maxwell
# Description：
"""

"""日志指标统计"""

import time
from collections import defaultdict, deque
from typing import Dict, Optional
from threading import Lock


class LogMetrics:
    """日志指标统计器"""

    def __init__(self, window_size: int = 60):
        """
        Args:
            window_size: 统计窗口大小（秒）
        """
        self.window_size = window_size
        self.lock = Lock()

        # 日志计数（按级别）
        self.counts: Dict[str, int] = defaultdict(int)

        # 时间序列（用于速率计算）
        self.timestamps: Dict[str, deque] = defaultdict(
            lambda: deque(maxlen=1000)
        )

        # 错误统计
        self.error_count = 0
        self.error_timestamps = deque(maxlen=1000)

    def record(self, level: str):
        """
        记录日志

        Args:
            level: 日志级别
        """
        with self.lock:
            current_time = time.time()

            # 更新计数
            self.counts[level] += 1
            self.timestamps[level].append(current_time)

            # 记录错误
            if level in ("ERROR", "CRITICAL"):
                self.error_count += 1
                self.error_timestamps.append(current_time)

    def get_counts(self) -> Dict[str, int]:
        """获取日志计数"""
        with self.lock:
            return dict(self.counts)

    def get_rate(self, level: str) -> float:
        """
        获取日志速率（每秒）

        Args:
            level: 日志级别

        Returns:
            每秒日志数
        """
        with self.lock:
            timestamps = self.timestamps[level]
            if not timestamps:
                return 0.0

            current_time = time.time()

            # 过滤窗口内的时间戳
            recent = [
                ts for ts in timestamps
                if current_time - ts <= self.window_size
            ]

            if not recent:
                return 0.0

            duration = current_time - recent[0]
            return len(recent) / duration if duration > 0 else 0.0

    def get_error_rate(self) -> float:
        """获取错误速率（每秒）"""
        with self.lock:
            if not self.error_timestamps:
                return 0.0

            current_time = time.time()

            # 过滤窗口内的时间戳
            recent = [
                ts for ts in self.error_timestamps
                if current_time - ts <= self.window_size
            ]

            if not recent:
                return 0.0

            duration = current_time - recent[0]
            return len(recent) / duration if duration > 0 else 0.0

    def check_error_threshold(self, threshold: int, window: int) -> bool:
        """
        检查错误阈值

        Args:
            threshold: 错误阈值
            window: 时间窗口（秒）

        Returns:
            True: 超过阈值
            False: 未超过阈值
        """
        with self.lock:
            current_time = time.time()

            # 统计窗口内的错误数
            recent_errors = sum(
                1 for ts in self.error_timestamps
                if current_time - ts <= window
            )

            return recent_errors >= threshold

    def get_summary(self) -> Dict[str, any]:
        """获取统计摘要"""
        with self.lock:
            return {
                "total_counts": dict(self.counts),
                "error_count": self.error_count,
                "error_rate": round(self.get_error_rate(), 2),
                "rates": {
                    level: round(self.get_rate(level), 2)
                    for level in self.counts.keys()
                },
            }


# 全局指标实例
_metrics_instance: Optional[LogMetrics] = None


def get_metrics() -> LogMetrics:
    """获取指标实例"""
    global _metrics_instance
    if _metrics_instance is None:
        _metrics_instance = LogMetrics()
    return _metrics_instance



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2024/9/29 08:01
# Author     ：Maxwell
# Description：
"""
import json
import aiohttp
from infoman.utils.log import logger


class RobotManager(object):

    @classmethod
    async def publish_message(cls, url, message):
        try:
            headers = {"Content-Type": "application/json"}
            json_data = {"msg_type": "text", "content": {"text": message}}

            async with aiohttp.ClientSession() as session:
                async with session.post(
                    url, headers=headers, data=json.dumps(json_data)
                ) as response:
                    if response.status == 200:
                        logger.info("RobotManager publish_message sent successfully")
                    else:
                        logger.info(
                            f"RobotManager publish_message. Status code: {response.status}"
                        )
                        response_text = await response.text()
                        logger.info(f"RobotManager publish_message: {response_text}")
        except Exception as e:
            logger.info(f"RobotManager error: {e}")



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/6/21 21:51
# Author     ：Maxwell
# Description：
"""
from infoman.service.exception.error import AppError
from infoman.config.settings import settings


class AppException(Exception):

    def __init__(self, error: AppError):
        self.error_code = error.code
        self.message = error.message_en
        if not settings.DEFAULT_LANGUAGE_IS_EN:
            self.message = error.message
        super().__init__(error.message)

    def with_content(self, place_holder, content):
        if self.message:
            self.message = self.message.replace(place_holder, content)
        return self



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/6/22 12:11
# Author     ：Maxwell
# Description：
"""
import time
import functools
from typing import Callable, TypeVar, Any, Optional, Union, Dict, List, Tuple
from infoman.logger import logger
from inspect import iscoroutinefunction

T = TypeVar("T")
F = TypeVar("F", bound=Callable[..., Any])


def cache(
    ttl: Optional[int] = 60,  # 缓存生存时间(秒)
    max_size: int = 128,  # 最大缓存条目数
    key_func: Optional[Callable] = None,  # 自定义缓存键生成函数
) -> Callable[[F], F]:
    """
    缓存装饰器 - 缓存函数结果以提高性能

    参数:
        ttl: 缓存生存时间(秒)，None表示永不过期
        max_size: 最大缓存条目数
        key_func: 自定义缓存键生成函数

    示例:
        @cache(ttl=300)  # 缓存5分钟
        def get_user_data(user_id):
            # 获取用户数据的代码
    """
    cache_dict: Dict[str, Tuple[Any, Optional[float]]] = {}

    def make_key(*args, **kwargs) -> str:
        """生成缓存键"""
        if key_func:
            return str(key_func(*args, **kwargs))

        # 默认键生成逻辑
        key_parts = [str(arg) for arg in args]
        key_parts.extend(f"{k}={v}" for k, v in sorted(kwargs.items()))
        return ":".join(key_parts)

    def is_expired(timestamp: Optional[float]) -> bool:
        """检查缓存是否过期"""
        if timestamp is None:  # 永不过期
            return False
        return time.time() > timestamp

    def cleanup_cache() -> None:
        """清理过期缓存"""
        if ttl is None:
            return

        expired_keys = [
            k
            for k, (_, exp_time) in cache_dict.items()
            if exp_time is not None and is_expired(exp_time)
        ]

        for k in expired_keys:
            del cache_dict[k]

    def decorator(func: F) -> F:
        @functools.wraps(func)
        async def async_wrapper(*args, **kwargs):
            # 清理过期缓存
            cleanup_cache()

            # 生成缓存键
            key = make_key(*args, **kwargs)

            # 检查缓存
            if key in cache_dict:
                value, exp_time = cache_dict[key]
                if not is_expired(exp_time):
                    return value

            # 缓存未命中，执行函数
            result = await func(*args, **kwargs)

            # 更新缓存
            if len(cache_dict) >= max_size:
                # 简单的LRU策略：删除第一个条目
                if cache_dict:
                    cache_dict.pop(next(iter(cache_dict)))

            expiration = (time.time() + ttl) if ttl is not None else None
            cache_dict[key] = (result, expiration)

            return result

        @functools.wraps(func)
        def sync_wrapper(*args, **kwargs):
            # 清理过期缓存
            cleanup_cache()

            # 生成缓存键
            key = make_key(*args, **kwargs)

            # 检查缓存
            if key in cache_dict:
                value, exp_time = cache_dict[key]
                if not is_expired(exp_time):
                    return value

            # 缓存未命中，执行函数
            result = func(*args, **kwargs)

            # 更新缓存
            if len(cache_dict) >= max_size:
                # 简单的LRU策略：删除第一个条目
                if cache_dict:
                    cache_dict.pop(next(iter(cache_dict)))

            expiration = (time.time() + ttl) if ttl is not None else None
            cache_dict[key] = (result, expiration)

            return result

        # 添加清除缓存的辅助方法
        def clear_cache():
            cache_dict.clear()

        if iscoroutinefunction(func):
            async_wrapper.clear_cache = clear_cache
            return async_wrapper
        else:
            sync_wrapper.clear_cache = clear_cache
            return sync_wrapper

    return decorator



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2024/1/5 16:25
# Author     ：Maxwell
# Description：
"""

import re
import json
from typing import Optional, Dict, List, Union


def extract_json_from_string(s: str) -> Optional[Union[Dict, List]]:
    if not s:
        return None

    json_pattern = re.compile(r"(\{[^{}]*}|$[^\[$]*])", re.DOTALL)
    candidates = []

    for match in json_pattern.finditer(s):
        candidate = match.group(0)
        try:
            if candidate.startswith("{") or candidate.startswith("["):
                candidates.append((match.start(), match.end(), candidate))
        except (IndexError, AttributeError):
            continue

    for start, end, candidate in candidates:
        if (
            candidate.startswith("{") and candidate.count("{") == candidate.count("}")
        ) or (
            candidate.startswith("[") and candidate.count("[") == candidate.count("]")
        ):
            try:
                parsed = json.loads(candidate)
                return parsed
            except json.JSONDecodeError:
                expanded = _expand_json_boundary(s, start, end)
                if expanded:
                    try:
                        return json.loads(expanded)
                    except json.JSONDecodeError:
                        continue
                continue

    return _robust_json_parse(s)


def _expand_json_boundary(s: str, start: int, end: int) -> Optional[str]:
    stack = []
    in_string = False
    escape = False

    for i in range(start, len(s)):
        c = s[i]

        if c == '"' and not escape:
            in_string = not in_string
        elif not in_string:
            if c == "{" or c == "[":
                stack.append(c)
            elif c == "}" and stack and stack[-1] == "{":
                stack.pop()
            elif c == "]" and stack and stack[-1] == "[":
                stack.pop()

        escape = c == "\\" and not escape

        if not stack and not in_string:
            return s[start : i + 1]

    return None


def _robust_json_parse(s: str) -> Optional[Union[Dict, List]]:
    stack = []
    in_string = False
    escape = False
    start_index = None
    structures = []

    for i, c in enumerate(s):
        if c == '"' and not escape:
            in_string = not in_string
        elif not in_string:
            if c == "{" or c == "[":
                if not stack:
                    start_index = i
                stack.append(c)
            elif c == "}" and stack and stack[-1] == "{":
                stack.pop()
                if not stack:
                    structures.append((start_index, i + 1))
            elif c == "]" and stack and stack[-1] == "[":
                stack.pop()
                if not stack:
                    structures.append((start_index, i + 1))

        escape = c == "\\" and not escape

    structures.sort(key=lambda x: x[1] - x[0], reverse=True)

    for start, end in structures:
        try:
            return json.loads(s[start:end])
        except json.JSONDecodeError:
            continue

    return None



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/12/23 17:59
# Author     ：Maxwell
# Description：
"""
import sys
import socket
from typing import Optional, cast
from loguru import logger
from infoman.config.log import get_log_config, LogConfig
from .formatters import get_console_format, get_file_format, serialize_json
from .handlers import LokiHandler
from .filters import (
    ModuleFilter,
    KeywordFilter,
    LevelFilter,
    RateLimitFilter,
    SanitizeFilter,
)
from .context import get_all_context
from .metrics import get_metrics


class LoggerManager:

    def __init__(self, config: Optional[LogConfig] = None):
        """
        初始化日志管理器

        Args:
            config: 日志配置（None 则自动加载）
        """
        self.config = config or get_log_config()
        self.is_initialized = False
        self._handler_ids = []

        # 获取主机名和应用名
        self.hostname = socket.gethostname()
        self.app_name = getattr(self.config, 'APP_NAME', 'infoman')

    def initialize(self):
        """初始化日志系统"""
        if self.is_initialized:
            logger.warning("日志系统已初始化，跳过重复初始化")
            return

        # 移除默认处理器
        logger.remove()

        # 配置日志处理器
        self._setup_console_handler()
        self._setup_file_handlers()
        self._setup_loki_handler()

        # 配置全局过滤器
        self._setup_filters()

        # 配置上下文注入
        self._setup_context_injection()

        # 配置指标统计
        if self.config.LOG_ENABLE_METRICS:
            self._setup_metrics()

        self.is_initialized = True

        logger.info(
            f"日志系统初始化完成 [ENV={self.config.ENV}, "
            f"LEVEL={self.config.LOG_LEVEL}, "
            f"FORMAT={self.config.LOG_FORMAT}]"
        )

    def _setup_console_handler(self):
        """配置控制台处理器"""
        if not self.config.LOG_ENABLE_CONSOLE:
            return

        console_format = get_console_format(self.config.LOG_FORMAT)
        handler_config = {
            "sink": sys.stderr,
            "level": self.config.LOG_LEVEL_CONSOLE or self.config.LOG_LEVEL,
            "format": console_format,
            "colorize": self.config.LOG_ENABLE_CONSOLE_COLOR,
            "backtrace": self.config.LOG_ENABLE_BACKTRACE,
            "diagnose": self.config.LOG_ENABLE_DIAGNOSE,
        }
        handler_id = logger.add(**handler_config)
        self._handler_ids.append(handler_id)

        logger.debug("控制台处理器已配置")

    def _setup_file_handlers(self):
        """配置文件处理器"""
        if not self.config.LOG_ENABLE_FILE:
            return

        file_format = get_file_format(self.config.LOG_FORMAT)

        # 通用文件配置
        common_config = {
            "format": file_format,
            "rotation": self.config.LOG_ROTATION,
            "retention": self.config.LOG_RETENTION,
            "compression": self.config.LOG_COMPRESSION,
            "encoding": self.config.LOG_FILE_ENCODING,
            "backtrace": self.config.LOG_ENABLE_BACKTRACE,
            "diagnose": self.config.LOG_ENABLE_DIAGNOSE,
            "enqueue": self.config.LOG_ENABLE_ASYNC,
        }

        # JSON 格式特殊处理
        if self.config.LOG_FORMAT == "json":
            common_config["serialize"] = True

        # ========== all.log（所有日志） ==========
        if self.config.LOG_ENABLE_ALL_FILE:
            handler_id = logger.add(
                sink=self.config.LOG_DIR / "all.log",
                level=cast(str, self.config.LOG_LEVEL_FILE or self.config.LOG_LEVEL),
                **common_config
            )
            self._handler_ids.append(handler_id)
            logger.debug("all.log 处理器已配置")

        # ========== 按级别分割日志 ==========
        if self.config.LOG_ENABLE_SPLIT_BY_LEVEL:
            # info.log（INFO 及以上）
            handler_id = logger.add(
                sink=self.config.LOG_DIR / "info.log",
                level="INFO",
                filter=lambda record: record["level"].name == "INFO",
                **common_config
            )
            self._handler_ids.append(handler_id)

            # warning.log（WARNING 及以上）
            handler_id = logger.add(
                sink=self.config.LOG_DIR / "warning.log",
                level="WARNING",
                filter=lambda record: record["level"].name == "WARNING",
                **common_config
            )
            self._handler_ids.append(handler_id)

            # error.log（ERROR 及以上）
            handler_id = logger.add(
                sink=self.config.LOG_DIR / "error.log",
                level="ERROR",
                filter=lambda record: record["level"].name in ("ERROR", "CRITICAL"),
                **common_config
            )
            self._handler_ids.append(handler_id)

            logger.debug("分级日志文件处理器已配置")

        # ========== debug.log（开发环境） ==========
        if self.config.LOG_ENABLE_DEBUG_FILE:
            handler_id = logger.add(
                sink=self.config.LOG_DIR / "debug.log",
                level="DEBUG",
                **common_config
            )
            self._handler_ids.append(handler_id)
            logger.debug("debug.log 处理器已配置")

    def _setup_loki_handler(self):
        """配置 Loki 处理器"""
        if not self.config.LOG_ENABLE_LOKI:
            return

        try:
            loki_handler = LokiHandler(
                url=self.config.LOG_LOKI_URL,
                labels=self.config.get_loki_labels(),
                batch_size=self.config.LOG_LOKI_BATCH_SIZE,
                flush_interval=self.config.LOG_LOKI_FLUSH_INTERVAL,
                timeout=self.config.LOG_LOKI_TIMEOUT,
                retry_times=self.config.LOG_LOKI_RETRY_TIMES,
                retry_backoff=self.config.LOG_LOKI_RETRY_BACKOFF,
                enable_fallback=self.config.LOG_LOKI_ENABLE_FALLBACK,
                fallback_file=self.config.LOG_DIR / self.config.LOG_LOKI_FALLBACK_FILE,
            )
            handler_id = logger.add(
                sink=loki_handler,
                level=cast(str, self.config.LOG_LEVEL_LOKI or self.config.LOG_LEVEL),
                format="{message}",  # Loki 处理器自己格式化
                enqueue=True,  # 异步推送
            )
            self._handler_ids.append(handler_id)

            logger.info(f"Loki 处理器已配置 [URL={self.config.LOG_LOKI_URL}]")

        except ImportError:
            logger.error("无法导入 LokiHandler，请检查 httpx 是否安装")
        except Exception as e:
            logger.error(f"配置 Loki 处理器失败: {e}")

    def _setup_filters(self):
        """配置全局过滤器"""
        # 模块过滤
        if self.config.LOG_FILTER_MODULES:
            module_filter = ModuleFilter(self.config.LOG_FILTER_MODULES)
            logger.add(
                lambda msg: None,  # 空处理器
                filter=module_filter,
            )
            logger.debug(f"模块过滤器已配置: {self.config.LOG_FILTER_MODULES}")

        # 关键词过滤
        if self.config.LOG_FILTER_KEYWORDS:
            keyword_filter = KeywordFilter(self.config.LOG_FILTER_KEYWORDS)
            logger.add(
                lambda msg: None,
                filter=keyword_filter,
            )
            logger.debug(f"关键词过滤器已配置: {self.config.LOG_FILTER_KEYWORDS}")

        # 速率限制
        if self.config.LOG_ENABLE_RATE_LIMIT:
            rate_limit_filter = RateLimitFilter(
                self.config.LOG_RATE_LIMIT_MESSAGES
            )
            logger.add(
                lambda msg: None,
                filter=rate_limit_filter,
            )
            logger.debug(
                f"速率限制过滤器已配置: "
                f"{self.config.LOG_RATE_LIMIT_MESSAGES} msg/s"
            )

        # 敏感信息脱敏
        if self.config.LOG_ENABLE_SANITIZE:
            sanitize_filter = SanitizeFilter(
                self.config.LOG_SANITIZE_FIELDS,
                self.config.LOG_SANITIZE_PATTERN,
            )
            logger.add(
                lambda msg: None,
                filter=sanitize_filter,
            )
            logger.debug(
                f"脱敏过滤器已配置: {len(self.config.LOG_SANITIZE_FIELDS)} 个字段"
            )

    def _setup_context_injection(self):
        """配置上下文注入"""
        if not self.config.LOG_ENABLE_CONTEXT:
            return

        # 配置上下文处理器
        def context_patcher(record):
            """注入上下文信息"""
            context = get_all_context()

            # 注入到 extra 字段
            record["extra"].update(context)

            # 注入环境信息
            record["extra"]["hostname"] = self.hostname
            record["extra"]["app_name"] = self.app_name
            record["extra"]["env"] = self.config.ENV

        logger.configure(patcher=context_patcher)
        logger.debug("上下文注入已配置")

    def _setup_metrics(self):
        """配置指标统计"""
        metrics = get_metrics()

        def metrics_sink(message):
            """指标统计处理器"""
            record = message.record
            level = record["level"].name
            metrics.record(level)

            # 检查错误阈值
            if metrics.check_error_threshold(
                    self.config.LOG_ERROR_THRESHOLD,
                    self.config.LOG_ERROR_WINDOW
            ):
                logger.warning(
                    f"错误日志超过阈值: "
                    f"{self.config.LOG_ERROR_THRESHOLD} 条/"
                    f"{self.config.LOG_ERROR_WINDOW} 秒"
                )

        handler_id = logger.add(
            sink=metrics_sink,
            level="DEBUG",
            format="{message}",
        )
        self._handler_ids.append(handler_id)

        logger.debug("指标统计已配置")

    def shutdown(self):
        """关闭日志系统"""
        if not self.is_initialized:
            return

        logger.info("正在关闭日志系统...")

        # 移除所有处理器
        for handler_id in self._handler_ids:
            try:
                logger.remove(handler_id)
            except ValueError:
                pass

        self._handler_ids.clear()
        self.is_initialized = False

        logger.info("日志系统已关闭")

    def reload(self):
        """重新加载日志配置"""
        logger.info("正在重新加载日志配置...")

        self.shutdown()

        from infoman.config.log import reload_log_config
        self.config = reload_log_config()

        self.initialize()

        logger.info("日志配置已重新加载")


# =================================================================
# 全局实例
# =================================================================

_logger_manager: Optional[LoggerManager] = None


def setup_logger(config: Optional[LogConfig] = None) -> LoggerManager:
    """
    设置日志系统

    Args:
        config: 日志配置（None 则自动加载）

    Returns:
        日志管理器实例
    """
    global _logger_manager

    if _logger_manager is None:
        _logger_manager = LoggerManager(config)
        _logger_manager.initialize()

    return _logger_manager


def get_logger_manager() -> Optional[LoggerManager]:
    """获取日志管理器实例"""
    return _logger_manager


def shutdown_logger():
    """关闭日志系统"""
    global _logger_manager

    if _logger_manager is not None:
        _logger_manager.shutdown()
        _logger_manager = None
